@@ -1,5 +1,5 @@
 # postgresql/psycopg2.py
-# Copyright (C) 2005-2012 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2013 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -10,7 +10,7 @@ Driver
 ------
 
 The psycopg2 driver is available at http://pypi.python.org/pypi/psycopg2/ .
-The dialect has several behaviors  which are specifically tailored towards compatibility 
+The dialect has several behaviors  which are specifically tailored towards compatibility
 with this module.
 
 Note that psycopg1 is **not** supported.
@@ -38,14 +38,34 @@ psycopg2-specific keyword arguments which are accepted by
 * *use_native_unicode* - Enable the usage of Psycopg2 "native unicode" mode
   per connection. True by default.
 
+Unix Domain Connections
+------------------------
+
+psycopg2 supports connecting via Unix domain connections.   When the ``host``
+portion of the URL is omitted, SQLAlchemy passes ``None`` to psycopg2,
+which specifies Unix-domain communication rather than TCP/IP communication::
+
+    create_engine("postgresql+psycopg2://user:password@/dbname")
+
+By default, the socket file used is to connect to a Unix-domain socket
+in ``/tmp``, or whatever socket directory was specified when PostgreSQL
+was built.  This value can be overridden by passing a pathname to psycopg2,
+using ``host`` as an additional keyword argument::
+
+    create_engine("postgresql+psycopg2://user:password@/dbname?host=/var/lib/postgresql")
+
+See also:
+
+`PQconnectdbParams <http://www.postgresql.org/docs/9.1/static/libpq-connect.html#LIBPQ-PQCONNECTDBPARAMS>`_
+
 Per-Statement/Connection Execution Options
 -------------------------------------------
 
-The following DBAPI-specific options are respected when used with 
+The following DBAPI-specific options are respected when used with
 :meth:`.Connection.execution_options`, :meth:`.Executable.execution_options`,
 :meth:`.Query.execution_options`, in addition to those not specific to DBAPIs:
 
-* isolation_level - Set the transaction isolation level for the lifespan of a 
+* isolation_level - Set the transaction isolation level for the lifespan of a
   :class:`.Connection` (can only be set on a connection, not a statement or query).
   This includes the options ``SERIALIZABLE``, ``READ COMMITTED``,
   ``READ UNCOMMITTED`` and ``REPEATABLE READ``.
@@ -59,8 +79,8 @@ By default, the psycopg2 driver uses the ``psycopg2.extensions.UNICODE``
 extension, such that the DBAPI receives and returns all strings as Python
 Unicode objects directly - SQLAlchemy passes these values through without
 change.   Psycopg2 here will encode/decode string values based on the
-current "client encoding" setting; by default this is the value in 
-the ``postgresql.conf`` file, which often defaults to ``SQL_ASCII``.   
+current "client encoding" setting; by default this is the value in
+the ``postgresql.conf`` file, which often defaults to ``SQL_ASCII``.
 Typically, this can be changed to ``utf-8``, as a more useful default::
 
     #client_encoding = sql_ascii # actually, defaults to database
@@ -70,26 +90,27 @@ Typically, this can be changed to ``utf-8``, as a more useful default::
 A second way to affect the client encoding is to set it within Psycopg2
 locally.   SQLAlchemy will call psycopg2's ``set_client_encoding()``
 method (see: http://initd.org/psycopg/docs/connection.html#connection.set_client_encoding)
-on all new connections based on the value passed to 
+on all new connections based on the value passed to
 :func:`.create_engine` using the ``client_encoding`` parameter::
 
     engine = create_engine("postgresql://user:pass@host/dbname", client_encoding='utf8')
 
 This overrides the encoding specified in the Postgresql client configuration.
-The psycopg2-specific ``client_encoding`` parameter to :func:`.create_engine` is new as of 
-SQLAlchemy 0.7.3.
+
+.. versionadded:: 0.7.3
+    The psycopg2-specific ``client_encoding`` parameter to :func:`.create_engine`.
 
 SQLAlchemy can also be instructed to skip the usage of the psycopg2
 ``UNICODE`` extension and to instead utilize it's own unicode encode/decode
-services, which are normally reserved only for those DBAPIs that don't 
-fully support unicode directly.  Passing ``use_native_unicode=False`` 
+services, which are normally reserved only for those DBAPIs that don't
+fully support unicode directly.  Passing ``use_native_unicode=False``
 to :func:`.create_engine` will disable usage of ``psycopg2.extensions.UNICODE``.
-SQLAlchemy will instead encode data itself into Python bytestrings on the way 
+SQLAlchemy will instead encode data itself into Python bytestrings on the way
 in and coerce from bytes on the way back,
-using the value of the :func:`.create_engine` ``encoding`` parameter, which 
+using the value of the :func:`.create_engine` ``encoding`` parameter, which
 defaults to ``utf-8``.
 SQLAlchemy's own unicode encode/decode functionality is steadily becoming
-obsolete as more DBAPIs support unicode fully along with the approach of 
+obsolete as more DBAPIs support unicode fully along with the approach of
 Python 3; in modern usage psycopg2 should be relied upon to handle unicode.
 
 Transactions
@@ -97,6 +118,8 @@ Transactions
 
 The psycopg2 dialect fully supports SAVEPOINT and two-phase commit operations.
 
+.. _psycopg2_isolation:
+
 Transaction Isolation Level
 ---------------------------
 
@@ -109,7 +132,7 @@ at the API level what level should be used.
 NOTICE logging
 ---------------
 
-The psycopg2 dialect will log Postgresql NOTICE messages via the 
+The psycopg2 dialect will log Postgresql NOTICE messages via the
 ``sqlalchemy.dialects.postgresql`` logger::
 
     import logging
@@ -197,8 +220,8 @@ class PGExecutionContext_psycopg2(PGExecutionContext):
                     (self.compiled and isinstance(self.compiled.statement, expression.Selectable) \
                     or \
                     (
-                        (not self.compiled or 
-                        isinstance(self.compiled.statement, expression._TextClause)) 
+                        (not self.compiled or
+                        isinstance(self.compiled.statement, expression._TextClause))
                         and self.statement and SERVER_SIDE_CURSOR_RE.match(self.statement))
                     )
                 )
@@ -226,7 +249,7 @@ class PGExecutionContext_psycopg2(PGExecutionContext):
 
     def _log_notices(self, cursor):
         for notice in cursor.connection.notices:
-            # NOTICE messages have a 
+            # NOTICE messages have a
             # newline character at the end
             logger.info(notice.rstrip())
 
@@ -268,7 +291,7 @@ class PGDialect_psycopg2(PGDialect):
         }
     )
 
-    def __init__(self, server_side_cursors=False, use_native_unicode=True, 
+    def __init__(self, server_side_cursors=False, use_native_unicode=True,
                         client_encoding=None, **kwargs):
         PGDialect.__init__(self, **kwargs)
         self.server_side_cursors = server_side_cursors
@@ -276,12 +299,12 @@ class PGDialect_psycopg2(PGDialect):
         self.supports_unicode_binds = use_native_unicode
         self.client_encoding = client_encoding
         if self.dbapi and hasattr(self.dbapi, '__version__'):
-            m = re.match(r'(\d+)\.(\d+)(?:\.(\d+))?', 
+            m = re.match(r'(\d+)\.(\d+)(?:\.(\d+))?',
                                 self.dbapi.__version__)
             if m:
                 self.psycopg2_version = tuple(
-                                            int(x) 
-                                            for x in m.group(1, 2, 3) 
+                                            int(x)
+                                            for x in m.group(1, 2, 3)
                                             if x is not None)
 
     @classmethod
@@ -293,8 +316,8 @@ class PGDialect_psycopg2(PGDialect):
     def _isolation_lookup(self):
         extensions = __import__('psycopg2.extensions').extensions
         return {
-            'READ COMMITTED':extensions.ISOLATION_LEVEL_READ_COMMITTED, 
-            'READ UNCOMMITTED':extensions.ISOLATION_LEVEL_READ_UNCOMMITTED, 
+            'READ COMMITTED':extensions.ISOLATION_LEVEL_READ_COMMITTED,
+            'READ UNCOMMITTED':extensions.ISOLATION_LEVEL_READ_UNCOMMITTED,
             'REPEATABLE READ':extensions.ISOLATION_LEVEL_REPEATABLE_READ,
             'SERIALIZABLE':extensions.ISOLATION_LEVEL_SERIALIZABLE
         }
@@ -305,9 +328,9 @@ class PGDialect_psycopg2(PGDialect):
         except KeyError:
             raise exc.ArgumentError(
                 "Invalid value '%s' for isolation_level. "
-                "Valid isolation levels for %s are %s" % 
+                "Valid isolation levels for %s are %s" %
                 (level, self.name, ", ".join(self._isolation_lookup))
-                ) 
+                )
 
         connection.set_isolation_level(level)
 
@@ -347,9 +370,10 @@ class PGDialect_psycopg2(PGDialect):
     def is_disconnect(self, e, connection, cursor):
         if isinstance(e, self.dbapi.OperationalError):
             # these error messages from libpq: interfaces/libpq/fe-misc.c.
-            # TODO: these are sent through gettext in libpq and we can't 
-            # check within other locales - consider using connection.closed 
-            return 'closed the connection' in str(e) or \
+            # TODO: these are sent through gettext in libpq and we can't
+            # check within other locales - consider using connection.closed
+            return 'terminating connection' in str(e) or \
+                    'closed the connection' in str(e) or \
                     'connection not open' in str(e) or \
                     'could not receive data from server' in str(e)
         elif isinstance(e, self.dbapi.InterfaceError):
@@ -357,7 +381,7 @@ class PGDialect_psycopg2(PGDialect):
             return 'connection already closed' in str(e) or \
                     'cursor already closed' in str(e)
         elif isinstance(e, self.dbapi.ProgrammingError):
-            # not sure where this path is originally from, it may 
+            # not sure where this path is originally from, it may
             # be obsolete.   It really says "losed", not "closed".
             return "losed the connection unexpectedly" in str(e)
         else:
