@@ -1,5 +1,5 @@
 # ext/horizontal_shard.py
-# Copyright (C) 2005-2012 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2013 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -9,12 +9,12 @@
 Defines a rudimental 'horizontal sharding' system which allows a Session to
 distribute queries and persistence operations across multiple databases.
 
-For a usage example, see the :ref:`examples_sharding` example included in 
-the source distrbution.
+For a usage example, see the :ref:`examples_sharding` example included in
+the source distribution.
 
 """
 
-import sqlalchemy.exceptions as sa_exc
+from sqlalchemy import exc as sa_exc
 from sqlalchemy import util
 from sqlalchemy.orm.session import Session
 from sqlalchemy.orm.query import Query
@@ -31,7 +31,7 @@ class ShardedQuery(Query):
     def set_shard(self, shard_id):
         """return a new query, limited to a single shard ID.
 
-        all subsequent operations with the returned query will 
+        all subsequent operations with the returned query will
         be against the single shard regardless of other state.
         """
 
@@ -40,20 +40,23 @@ class ShardedQuery(Query):
         return q
 
     def _execute_and_instances(self, context):
-        if self._shard_id is not None:
-            result = self.session.connection(
+        def iter_for_shard(shard_id):
+            context.attributes['shard_id'] = shard_id
+            result = self._connection_from_session(
                             mapper=self._mapper_zero(),
-                            shard_id=self._shard_id).execute(context.statement, self._params)
+                            shard_id=shard_id).execute(
+                                                context.statement,
+                                                self._params)
             return self.instances(result, context)
+
+        if self._shard_id is not None:
+            return iter_for_shard(self._shard_id)
         else:
             partial = []
             for shard_id in self.query_chooser(self):
-                result = self.session.connection(
-                            mapper=self._mapper_zero(),
-                            shard_id=shard_id).execute(context.statement, self._params)
-                partial = partial + list(self.instances(result, context))
+                partial.extend(iter_for_shard(shard_id))
 
-            # if some kind of in memory 'sorting' 
+            # if some kind of in memory 'sorting'
             # were done, this is where it would happen
             return iter(partial)
 
@@ -70,7 +73,7 @@ class ShardedQuery(Query):
                 return None
 
 class ShardedSession(Session):
-    def __init__(self, shard_chooser, id_chooser, query_chooser, shards=None, 
+    def __init__(self, shard_chooser, id_chooser, query_chooser, shards=None,
                  query_cls=ShardedQuery, **kwargs):
         """Construct a ShardedSession.
 
@@ -98,7 +101,7 @@ class ShardedSession(Session):
         self.id_chooser = id_chooser
         self.query_chooser = query_chooser
         self.__binds = {}
-        self._mapper_flush_opts = {'connection_callable':self.connection}
+        self.connection_callable = self.connection
         if shards is not None:
             for k in shards:
                 self.bind_shard(k, shards[k])
@@ -110,8 +113,8 @@ class ShardedSession(Session):
         if self.transaction is not None:
             return self.transaction.connection(mapper, shard_id=shard_id)
         else:
-            return self.get_bind(mapper, 
-                                shard_id=shard_id, 
+            return self.get_bind(mapper,
+                                shard_id=shard_id,
                                 instance=instance).contextual_connect(**kwargs)
 
     def get_bind(self, mapper, shard_id=None, instance=None, clause=None, **kw):
@@ -122,3 +125,4 @@ class ShardedSession(Session):
     def bind_shard(self, shard_id, bind):
         self.__binds[shard_id] = bind
 
+
