@@ -1,5 +1,5 @@
 # orm/interfaces.py
-# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2013 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -21,7 +21,7 @@ from itertools import chain
 from sqlalchemy import exc as sa_exc
 from sqlalchemy import util
 from sqlalchemy.sql import operators
-deque = util.importlater('collections').deque
+deque = __import__('collections').deque
 
 mapperutil = util.importlater('sqlalchemy.orm', 'util')
 
@@ -61,6 +61,13 @@ class MapperProperty(object):
     attribute, as well as that attribute as it appears on individual
     instances of the class, including attribute instrumentation,
     attribute access, loading behavior, and dependency calculations.
+
+    The most common occurrences of :class:`.MapperProperty` are the
+    mapped :class:`.Column`, which is represented in a mapping as
+    an instance of :class:`.ColumnProperty`,
+    and a reference to another class produced by :func:`.relationship`,
+    represented in the mapping as an instance of :class:`.RelationshipProperty`.
+
     """
 
     cascade = ()
@@ -80,7 +87,7 @@ class MapperProperty(object):
 
         pass
 
-    def create_row_processor(self, context, path, reduced_path, 
+    def create_row_processor(self, context, path, reduced_path,
                                             mapper, row, adapter):
         """Return a 3-tuple consisting of three row processing functions.
 
@@ -244,8 +251,7 @@ class PropComparator(operators.ColumnOperators):
             query.join(Company.employees.of_type(Engineer)).\\
                filter(Engineer.name=='foo')
 
-        \class_
-            a class or mapper indicating that criterion will be against
+        :param \class_: a class or mapper indicating that criterion will be against
             this specific subclass.
 
 
@@ -257,13 +263,16 @@ class PropComparator(operators.ColumnOperators):
         """Return true if this collection contains any member that meets the
         given criterion.
 
-        criterion
-          an optional ClauseElement formulated against the member class' table
-          or attributes.
+        The usual implementation of ``any()`` is
+        :meth:`.RelationshipProperty.Comparator.any`.
+
+        :param criterion: an optional ClauseElement formulated against the
+          member class' table or attributes.
+
+        :param \**kwargs: key/value pairs corresponding to member class attribute
+          names which will be compared via equality to the corresponding
+          values.
 
-        \**kwargs
-          key/value pairs corresponding to member class attribute names which
-          will be compared via equality to the corresponding values.
         """
 
         return self.operate(PropComparator.any_op, criterion, **kwargs)
@@ -272,13 +281,16 @@ class PropComparator(operators.ColumnOperators):
         """Return true if this element references a member which meets the
         given criterion.
 
-        criterion
-          an optional ClauseElement formulated against the member class' table
-          or attributes.
+        The usual implementation of ``has()`` is
+        :meth:`.RelationshipProperty.Comparator.has`.
+
+        :param criterion: an optional ClauseElement formulated against the
+          member class' table or attributes.
+
+        :param \**kwargs: key/value pairs corresponding to member class attribute
+          names which will be compared via equality to the corresponding
+          values.
 
-        \**kwargs
-          key/value pairs corresponding to member class attribute names which
-          will be compared via equality to the corresponding values.
         """
 
         return self.operate(PropComparator.has_op, criterion, **kwargs)
@@ -294,16 +306,24 @@ class StrategizedProperty(MapperProperty):
 
     """
 
+    strategy_wildcard_key = None
+
     def _get_context_strategy(self, context, reduced_path):
         key = ('loaderstrategy', reduced_path)
+        cls = None
         if key in context.attributes:
             cls = context.attributes[key]
+        elif self.strategy_wildcard_key:
+            key = ('loaderstrategy', (self.strategy_wildcard_key,))
+            if key in context.attributes:
+                cls = context.attributes[key]
+
+        if cls:
             try:
                 return self._strategies[cls]
             except KeyError:
                 return self.__init_strategy(cls)
-        else:
-            return self.strategy
+        return self.strategy
 
     def _get_strategy(self, cls):
         try:
@@ -317,12 +337,12 @@ class StrategizedProperty(MapperProperty):
 
     def setup(self, context, entity, path, reduced_path, adapter, **kwargs):
         self._get_context_strategy(context, reduced_path + (self.key,)).\
-                    setup_query(context, entity, path, 
+                    setup_query(context, entity, path,
                                     reduced_path, adapter, **kwargs)
 
     def create_row_processor(self, context, path, reduced_path, mapper, row, adapter):
         return self._get_context_strategy(context, reduced_path + (self.key,)).\
-                    create_row_processor(context, path, 
+                    create_row_processor(context, path,
                                     reduced_path, mapper, row, adapter)
 
     def do_init(self):
@@ -345,7 +365,7 @@ def serialize_path(path):
         return None
 
     return zip(
-        [m.class_ for m in [path[i] for i in range(0, len(path), 2)]], 
+        [m.class_ for m in [path[i] for i in range(0, len(path), 2)]],
         [path[i] for i in range(1, len(path), 2)] + [None]
     )
 
@@ -362,7 +382,7 @@ class MapperOption(object):
     """Describe a modification to a Query."""
 
     propagate_to_loaders = False
-    """if True, indicate this option should be carried along 
+    """if True, indicate this option should be carried along
     Query object generated by scalar or object lazy loaders.
     """
 
@@ -380,7 +400,8 @@ class MapperOption(object):
 
 class PropertyOption(MapperOption):
     """A MapperOption that is applied to a property off the mapper or
-    one of its child mappers, identified by a dot-separated key. """
+    one of its child mappers, identified by a dot-separated key
+    or list of class-bound attributes. """
 
     def __init__(self, key, mapper=None):
         self.key = key
@@ -443,9 +464,9 @@ class PropertyOption(MapperOption):
                 else:
                     raise sa_exc.ArgumentError(
                         "Can't find property '%s' on any entity "
-                        "specified in this Query.  Note the full path " 
-                        "from root (%s) to target entity must be specified." 
-                        % (token, ",".join(str(x) for 
+                        "specified in this Query.  Note the full path "
+                        "from root (%s) to target entity must be specified."
+                        % (token, ",".join(str(x) for
                             x in query._mapper_entities))
                     )
             else:
@@ -473,7 +494,7 @@ class PropertyOption(MapperOption):
         l = []
         mappers = []
 
-        # _current_path implies we're in a 
+        # _current_path implies we're in a
         # secondary load with an existing path
         current_path = list(query._current_path)
 
@@ -481,6 +502,9 @@ class PropertyOption(MapperOption):
         while tokens:
             token = tokens.popleft()
             if isinstance(token, basestring):
+                # wildcard token
+                if token.endswith(':*'):
+                    return [(token,)], []
                 sub_tokens = token.split(".", 1)
                 token = sub_tokens[0]
                 tokens.extendleft(sub_tokens[1:])
@@ -496,16 +520,16 @@ class PropertyOption(MapperOption):
 
                 if not entity:
                     entity = self._find_entity_basestring(
-                                        query, 
-                                        token, 
+                                        query,
+                                        token,
                                         raiseerr)
                     if entity is None:
                         return [], []
                     path_element = entity.path_entity
                     mapper = entity.mapper
                 mappers.append(mapper)
-                if mapper.has_property(token):
-                    prop = mapper.get_property(token)
+                if hasattr(mapper.class_, token):
+                    prop = getattr(mapper.class_, token).property
                 else:
                     if raiseerr:
                         raise sa_exc.ArgumentError(
@@ -531,8 +555,8 @@ class PropertyOption(MapperOption):
                 if not entity:
                     entity = self._find_entity_prop_comparator(
                                             query,
-                                            prop.key, 
-                                            token.parententity, 
+                                            prop.key,
+                                            token.parententity,
                                             raiseerr)
                     if not entity:
                         return [], []
@@ -544,7 +568,12 @@ class PropertyOption(MapperOption):
                         "mapper option expects "
                         "string key or list of attributes")
             assert prop is not None
+            if raiseerr and not prop.parent.common_parent(mapper):
+                raise sa_exc.ArgumentError("Attribute '%s' does not "
+                            "link from element '%s'" % (token, path_element))
+
             path = build_path(path_element, prop.key, path)
+
             l.append(path)
             if getattr(token, '_of_type', None):
                 path_element = mapper = token._of_type
@@ -556,11 +585,9 @@ class PropertyOption(MapperOption):
                         "refer to a mapped entity" %
                         (token, entity)
                     )
-            if path_element:
-                path_element = path_element
 
         if current_path:
-            # ran out of tokens before 
+            # ran out of tokens before
             # current_path was exhausted.
             assert not tokens
             return [], []
@@ -603,9 +630,9 @@ def _reduce_path(path):
     of the mapper referenced by Mapper.prop1.
 
     """
-    return tuple([i % 2 != 0 and 
-                    element or 
-                    getattr(element, 'base_mapper', element) 
+    return tuple([i % 2 != 0 and
+                    element or
+                    getattr(element, 'base_mapper', element)
                     for i, element in enumerate(path)])
 
 class LoaderStrategy(object):
@@ -651,7 +678,7 @@ class LoaderStrategy(object):
     def setup_query(self, context, entity, path, reduced_path, adapter, **kwargs):
         pass
 
-    def create_row_processor(self, context, path, reduced_path, mapper, 
+    def create_row_processor(self, context, path, reduced_path, mapper,
                                 row, adapter):
         """Return row processing functions which fulfill the contract
         specified by MapperProperty.create_row_processor.
