@@ -1,45 +1,47 @@
-from sqlalchemy.test.testing import eq_
+from test.lib.testing import eq_
 from sqlalchemy.orm import mapper, relationship, create_session, \
-    clear_mappers, sessionmaker, class_mapper
+    clear_mappers, sessionmaker
 from sqlalchemy.orm.mapper import _mapper_registry
 from sqlalchemy.orm.session import _sessions
-from sqlalchemy.util import jython
 import operator
-from sqlalchemy.test import testing, engines
+from test.lib import testing, engines
 from sqlalchemy import MetaData, Integer, String, ForeignKey, \
-    PickleType, create_engine, Unicode
-from sqlalchemy.test.schema import Table, Column
+    PickleType, Unicode, select
+from test.lib.schema import Table, Column
 import sqlalchemy as sa
 from sqlalchemy.sql import column
 from sqlalchemy.processors import to_decimal_processor_factory, \
     to_unicode_processor_factory
-from sqlalchemy.test.util import gc_collect
-from decimal import Decimal as _python_Decimal
+from test.lib.util import gc_collect
+from sqlalchemy.util.compat import decimal
 import gc
 import weakref
-from test.orm import _base
+from test.lib import fixtures
 
-if jython:
-    from nose import SkipTest
-    raise SkipTest("Profiling not supported on this platform")
-
-
-class A(_base.ComparableEntity):
+class A(fixtures.ComparableEntity):
     pass
-class B(_base.ComparableEntity):
+class B(fixtures.ComparableEntity):
     pass
 
 def profile_memory(func):
     # run the test 50 times.  if length of gc.get_objects()
     # keeps growing, assert false
 
+    def get_objects_skipping_sqlite_issue():
+        # pysqlite keeps adding weakref objects which only
+        # get reset after 220 iterations, which is too long
+        # to run lots of these tests, so just filter them
+        # out.
+        return [o for o in gc.get_objects()
+                if not isinstance(o, weakref.ref)]
+
     def profile(*args):
         gc_collect()
         samples = [0 for x in range(0, 50)]
         for x in range(0, 50):
             func(*args)
             gc_collect()
-            samples[x] = len(gc.get_objects())
+            samples[x] = len(get_objects_skipping_sqlite_issue())
 
         print "sample gc sizes:", samples
 
@@ -68,13 +70,15 @@ def assert_no_mappers():
     gc_collect()
     assert len(_mapper_registry) == 0
 
-class EnsureZeroed(_base.ORMTest):
+class EnsureZeroed(fixtures.ORMTest):
     def setup(self):
         _sessions.clear()
         _mapper_registry.clear()
 
 class MemUsageTest(EnsureZeroed):
 
+    __requires__ = 'cpython',
+
     # ensure a pure growing test trips the assertion
     @testing.fails_if(lambda: True)
     def test_fixture(self):
@@ -143,6 +147,18 @@ class MemUsageTest(EnsureZeroed):
         del m1, m2, m3
         assert_no_mappers()
 
+    def test_sessionmaker(self):
+        @profile_memory
+        def go():
+            sessmaker = sessionmaker(bind=testing.db)
+            sess = sessmaker()
+            r = sess.execute(select([1]))
+            r.close()
+            sess.close()
+            del sess
+            del sessmaker
+        go()
+
     @testing.crashes('sqlite', ':memory: connection not suitable here')
     def test_orm_many_engines(self):
         metadata = MetaData(testing.db)
@@ -176,7 +192,8 @@ class MemUsageTest(EnsureZeroed):
         def go():
             engine = engines.testing_engine(
                                 options={'logging_name':'FOO',
-                                        'pool_logging_name':'BAR'}
+                                        'pool_logging_name':'BAR',
+                                        'use_reaper':False}
                                     )
             sess = create_session(bind=engine)
 
@@ -211,6 +228,35 @@ class MemUsageTest(EnsureZeroed):
         del m1, m2, m3
         assert_no_mappers()
 
+    def test_ad_hoc_types(self):
+        """test storage of bind processors, result processors
+        in dialect-wide registry."""
+
+        from sqlalchemy.dialects import mysql, postgresql, sqlite
+        from sqlalchemy import types
+
+        eng = engines.testing_engine()
+        for args in (
+            (types.Integer, ),
+            (types.String, ),
+            (types.PickleType, ),
+            (types.Enum, 'a', 'b', 'c'),
+            (sqlite.DATETIME, ),
+            (postgresql.ENUM, 'a', 'b', 'c'),
+            (types.Interval, ),
+            (postgresql.INTERVAL, ),
+            (mysql.VARCHAR, ),
+        ):
+            @profile_memory
+            def go():
+                type_ = args[0](*args[1:])
+                bp = type_._cached_bind_processor(eng.dialect)
+                rp = type_._cached_result_processor(eng.dialect, 0)
+            go()
+
+        assert not eng.dialect._type_memos
+
+
     def test_many_updates(self):
         metadata = MetaData(testing.db)
 
@@ -241,7 +287,7 @@ class MemUsageTest(EnsureZeroed):
             x = counter[0]
             dec = 10
             while dec > 0:
-                # trying to count in binary here, 
+                # trying to count in binary here,
                 # works enough to trip the test case
                 if pow(2, dec) < x:
                     setattr(w1, 'col%d' % dec, counter[0])
@@ -256,11 +302,6 @@ class MemUsageTest(EnsureZeroed):
         finally:
             metadata.drop_all()
 
-    @testing.fails_if(lambda : testing.db.dialect.name == 'sqlite' \
-                      and testing.db.dialect.dbapi.version_info >= (2,
-                      5),
-                      'Newer pysqlites generate warnings here too and '
-                      'have similar issues.')
     def test_unicode_warnings(self):
         metadata = MetaData(testing.db)
         table1 = Table('mytable', metadata, Column('col1', Integer,
@@ -359,7 +400,7 @@ class MemUsageTest(EnsureZeroed):
 
         @profile_memory
         def go():
-            class A(_base.ComparableEntity):
+            class A(fixtures.ComparableEntity):
                 pass
             class B(A):
                 pass
@@ -425,13 +466,13 @@ class MemUsageTest(EnsureZeroed):
 
         @profile_memory
         def go():
-            class A(_base.ComparableEntity):
+            class A(fixtures.ComparableEntity):
                 pass
-            class B(_base.ComparableEntity):
+            class B(fixtures.ComparableEntity):
                 pass
 
             mapper(A, table1, properties={
-                'bs':relationship(B, secondary=table3, 
+                'bs':relationship(B, secondary=table3,
                                     backref='as', order_by=table3.c.t1)
             })
             mapper(B, table2)
@@ -470,12 +511,24 @@ class MemUsageTest(EnsureZeroed):
             metadata.drop_all()
         assert_no_mappers()
 
+    @testing.provide_metadata
+    def test_key_fallback_result(self):
+        e = testing.db
+        m = self.metadata
+        t = Table('t', m, Column('x', Integer), Column('y', Integer))
+        m.create_all(e)
+        e.execute(t.insert(), {"x":1, "y":1})
+        @profile_memory
+        def go():
+            r = e.execute(t.alias().select())
+            for row in r:
+                row[t.c.x]
+        go()
+
     # fails on newer versions of pysqlite due to unusual memory behvior
     # in pysqlite itself. background at:
     # http://thread.gmane.org/gmane.comp.python.db.pysqlite.user/2290
 
-    @testing.fails_if(lambda : testing.db.dialect.name == 'sqlite' \
-                      and testing.db.dialect.dbapi.version > '2.5')
     def test_join_cache(self):
         metadata = MetaData(testing.db)
         table1 = Table('table1', metadata, Column('id', Integer,
@@ -517,7 +570,7 @@ class MemUsageTest(EnsureZeroed):
         table1 = Table("mytable", metadata,
             Column('col1', Integer, primary_key=True,
                                 test_needs_autoincrement=True),
-            Column('col2', PickleType(comparator=operator.eq))
+            Column('col2', PickleType(comparator=operator.eq, mutable=True))
             )
 
         class Foo(object):
@@ -580,7 +633,7 @@ class MemUsageTest(EnsureZeroed):
     def test_DecimalResultProcessor_process(self):
         @profile_memory
         def go():
-            to_decimal_processor_factory(_python_Decimal, 10)(1.2)
+            to_decimal_processor_factory(decimal.Decimal, 10)(1.2)
         go()
 
     @testing.requires.cextensions
