@@ -63,6 +63,96 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
                             'n WHERE sometable.somecolumn = '
                             ':somecolumn_1', dict(somecolumn=10))
 
+    def test_insert_hint(self):
+        t = table('sometable', column('somecolumn'))
+        for targ in (None, t):
+            for darg in ("*", "mssql"):
+                self.assert_compile(
+                    t.insert().
+                        values(somecolumn="x").
+                        with_hint("WITH (PAGLOCK)",
+                            selectable=targ,
+                            dialect_name=darg),
+                    "INSERT INTO sometable WITH (PAGLOCK) "
+                    "(somecolumn) VALUES (:somecolumn)"
+                )
+
+    def test_update_hint(self):
+        t = table('sometable', column('somecolumn'))
+        for targ in (None, t):
+            for darg in ("*", "mssql"):
+                self.assert_compile(
+                    t.update().where(t.c.somecolumn=="q").
+                            values(somecolumn="x").
+                            with_hint("WITH (PAGLOCK)",
+                                    selectable=targ,
+                                    dialect_name=darg),
+                    "UPDATE sometable WITH (PAGLOCK) "
+                    "SET somecolumn=:somecolumn "
+                    "WHERE sometable.somecolumn = :somecolumn_1"
+                )
+
+    def test_update_exclude_hint(self):
+        t = table('sometable', column('somecolumn'))
+        self.assert_compile(
+            t.update().where(t.c.somecolumn=="q").
+                values(somecolumn="x").
+                with_hint("XYZ", "mysql"),
+            "UPDATE sometable SET somecolumn=:somecolumn "
+            "WHERE sometable.somecolumn = :somecolumn_1"
+        )
+
+    def test_delete_hint(self):
+        t = table('sometable', column('somecolumn'))
+        for targ in (None, t):
+            for darg in ("*", "mssql"):
+                self.assert_compile(
+                    t.delete().where(t.c.somecolumn=="q").
+                            with_hint("WITH (PAGLOCK)",
+                                    selectable=targ,
+                                    dialect_name=darg),
+                    "DELETE FROM sometable WITH (PAGLOCK) "
+                    "WHERE sometable.somecolumn = :somecolumn_1"
+                )
+
+    def test_delete_exclude_hint(self):
+        t = table('sometable', column('somecolumn'))
+        self.assert_compile(
+            t.delete().\
+                where(t.c.somecolumn=="q").\
+                with_hint("XYZ", dialect_name="mysql"),
+            "DELETE FROM sometable WHERE "
+            "sometable.somecolumn = :somecolumn_1"
+        )
+
+    def test_update_from_hint(self):
+        t = table('sometable', column('somecolumn'))
+        t2 = table('othertable', column('somecolumn'))
+        for darg in ("*", "mssql"):
+            self.assert_compile(
+                t.update().where(t.c.somecolumn==t2.c.somecolumn).
+                        values(somecolumn="x").
+                        with_hint("WITH (PAGLOCK)",
+                                selectable=t2,
+                                dialect_name=darg),
+                "UPDATE sometable SET somecolumn=:somecolumn "
+                "FROM sometable, othertable WITH (PAGLOCK) "
+                "WHERE sometable.somecolumn = othertable.somecolumn"
+            )
+
+    # TODO: not supported yet.
+    #def test_delete_from_hint(self):
+    #    t = table('sometable', column('somecolumn'))
+    #    t2 = table('othertable', column('somecolumn'))
+    #    for darg in ("*", "mssql"):
+    #        self.assert_compile(
+    #            t.delete().where(t.c.somecolumn==t2.c.somecolumn).
+    #                    with_hint("WITH (PAGLOCK)",
+    #                            selectable=t2,
+    #                            dialect_name=darg),
+    #            ""
+    #        )
+
     # TODO: should this be for *all* MS-SQL dialects ?
     def test_mxodbc_binds(self):
         """mxodbc uses MS-SQL native binds, which aren't allowed in
@@ -73,7 +163,7 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
 
         for expr, compile in [
             (
-                select([literal("x"), literal("y")]), 
+                select([literal("x"), literal("y")]),
                 "SELECT 'x' AS anon_1, 'y' AS anon_2",
             ),
             (
@@ -337,14 +427,17 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
 
         s = select([t]).where(t.c.x==5).order_by(t.c.y).offset(20)
 
-        self.assert_compile(
-            s,
-            "SELECT anon_1.x, anon_1.y FROM (SELECT t.x AS x, t.y "
-            "AS y, ROW_NUMBER() OVER (ORDER BY t.y) AS "
-            "mssql_rn FROM t WHERE t.x = :x_1) AS "
-            "anon_1 WHERE mssql_rn > :mssql_rn_1",
-            checkparams={u'mssql_rn_1': 20, u'x_1': 5}
-        )
+        # test that the select is not altered with subsequent compile
+        # calls
+        for i in xrange(2):
+            self.assert_compile(
+                s,
+                "SELECT anon_1.x, anon_1.y FROM (SELECT t.x AS x, t.y "
+                "AS y, ROW_NUMBER() OVER (ORDER BY t.y) AS "
+                "mssql_rn FROM t WHERE t.x = :x_1) AS "
+                "anon_1 WHERE mssql_rn > :mssql_rn_1",
+                checkparams={u'mssql_rn_1': 20, u'x_1': 5}
+            )
 
     def test_limit_offset_using_window(self):
         t = table('t', column('x', Integer), column('y', Integer))
@@ -362,6 +455,27 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             checkparams={u'mssql_rn_1': 20, u'mssql_rn_2': 30, u'x_1': 5}
         )
 
+    def test_limit_offset_with_correlated_order_by(self):
+        t1 = table('t1', column('x', Integer), column('y', Integer))
+        t2 = table('t2', column('x', Integer), column('y', Integer))
+
+        order_by = select([t2.c.y]).where(t1.c.x == t2.c.x).as_scalar()
+        s = select([t1]).where(t1.c.x == 5).order_by(order_by) \
+            .limit(10).offset(20)
+
+        self.assert_compile(
+            s,
+            "SELECT anon_1.x, anon_1.y "
+            "FROM (SELECT t1.x AS x, t1.y AS y, "
+            "ROW_NUMBER() OVER (ORDER BY "
+            "(SELECT t2.y FROM t2 WHERE t1.x = t2.x)"
+            ") AS mssql_rn "
+            "FROM t1 "
+            "WHERE t1.x = :x_1) AS anon_1 "
+            "WHERE mssql_rn > :mssql_rn_1 AND mssql_rn <= :mssql_rn_2",
+            checkparams={u'mssql_rn_1': 20, u'mssql_rn_2': 30, u'x_1': 5}
+        )
+
     def test_limit_zero_offset_using_window(self):
         t = table('t', column('x', Integer), column('y', Integer))
 
@@ -603,8 +717,8 @@ class ReflectionTest(fixtures.TestBase, ComparesTables):
     def test_indexes_cols_with_commas(self):
         metadata = self.metadata
 
-        t1 = Table('t', metadata, 
-                        Column('x, col', Integer, key='x'), 
+        t1 = Table('t', metadata,
+                        Column('x, col', Integer, key='x'),
                         Column('y', Integer)
                     )
         Index('foo', t1.c.x, t1.c.y)
@@ -622,7 +736,7 @@ class ReflectionTest(fixtures.TestBase, ComparesTables):
     def test_indexes_cols_with_spaces(self):
         metadata = self.metadata
 
-        t1 = Table('t', metadata, Column('x col', Integer, key='x'), 
+        t1 = Table('t', metadata, Column('x col', Integer, key='x'),
                                     Column('y', Integer))
         Index('foo', t1.c.x, t1.c.y)
         metadata.create_all()
@@ -660,7 +774,8 @@ class QueryUnicodeTest(fixtures.TestBase):
         finally:
             meta.drop_all()
 
-class QueryTest(fixtures.TestBase):
+from test.lib.assertsql import ExactSQL
+class QueryTest(testing.AssertsExecutionResults, fixtures.TestBase):
     __only_on__ = 'mssql'
 
     def test_fetchid_trigger(self):
@@ -671,14 +786,14 @@ class QueryTest(fixtures.TestBase):
         case of a table having an identity (autoincrement)
         primary key column, and which also has a trigger configured
         to fire upon each insert and subsequently perform an
-        insert into a different table. 
+        insert into a different table.
 
         SQLALchemy's MSSQL dialect by default will attempt to
         use an OUTPUT_INSERTED clause, which in this case will
         raise the following error:
 
-        ProgrammingError: (ProgrammingError) ('42000', 334, 
-        "[Microsoft][SQL Server Native Client 10.0][SQL Server]The 
+        ProgrammingError: (ProgrammingError) ('42000', 334,
+        "[Microsoft][SQL Server Native Client 10.0][SQL Server]The
         target table 't1' of the DML statement cannot have any enabled
         triggers if the statement contains an OUTPUT clause without
         INTO clause.", 7748) 'INSERT INTO t1 (descr) OUTPUT inserted.id
@@ -705,7 +820,7 @@ class QueryTest(fixtures.TestBase):
                 # though the ExecutionContext will still have a
                 # _select_lastrowid, so the SELECT SCOPE_IDENTITY() will
                 # hopefully be called instead.
-                implicit_returning = False 
+                implicit_returning = False
                 )
         t2 = Table('t2', meta,
                 Column('id', Integer, Sequence('fred', 200, 1),
@@ -728,11 +843,32 @@ class QueryTest(fixtures.TestBase):
             con.execute("""drop trigger paj""")
             meta.drop_all()
 
+    @testing.provide_metadata
+    def test_disable_scope_identity(self):
+        engine = engines.testing_engine(options={"use_scope_identity":False})
+        metadata = self.metadata
+        metadata.bind = engine
+        t1 = Table('t1', metadata,
+                Column('id', Integer, primary_key=True),
+                implicit_returning=False
+        )
+        metadata.create_all()
+
+        self.assert_sql_execution(
+                testing.db,
+                lambda: engine.execute(t1.insert()),
+                ExactSQL("INSERT INTO t1 DEFAULT VALUES"),
+                # we dont have an event for
+                # "SELECT @@IDENTITY" part here.
+                # this will be in 0.8 with #2459
+        )
+        assert not engine.dialect.use_scope_identity
+
     def test_insertid_schema(self):
         meta = MetaData(testing.db)
         con = testing.db.connect()
         con.execute('create schema paj')
-        tbl = Table('test', meta, 
+        tbl = Table('test', meta,
                     Column('id', Integer, primary_key=True), schema='paj')
         tbl.create()
         try:
@@ -893,12 +1029,12 @@ class MatchTest(fixtures.TestBase, AssertsCompiledSQL):
             Column('category_id', Integer, ForeignKey('cattable.id')),
             PrimaryKeyConstraint('id', name='PK_matchtable'),
             )
-        DDL("""CREATE FULLTEXT INDEX 
-                       ON cattable (description) 
+        DDL("""CREATE FULLTEXT INDEX
+                       ON cattable (description)
                        KEY INDEX PK_cattable""").execute_at('after-create'
                 , matchtable)
-        DDL("""CREATE FULLTEXT INDEX 
-                       ON matchtable (title) 
+        DDL("""CREATE FULLTEXT INDEX
+                       ON matchtable (title)
                        KEY INDEX PK_matchtable""").execute_at('after-create'
                 , matchtable)
         metadata.create_all()
@@ -1013,8 +1149,9 @@ class ParseConnectTest(fixtures.TestBase, AssertsCompiledSQL):
             url.make_url('mssql://username:password@mydsn/?LANGUAGE=us_'
                          'english&foo=bar')
         connection = dialect.create_connect_args(u)
-        eq_([['dsn=mydsn;UID=username;PWD=password;LANGUAGE=us_english;'
-            'foo=bar'], {}], connection)
+        dsn_string = connection[0][0]
+        assert ";LANGUAGE=us_english" in dsn_string
+        assert ";foo=bar" in dsn_string
 
     def test_pyodbc_connect(self):
         u = url.make_url('mssql://username:password@hostspec/database')
@@ -1085,7 +1222,7 @@ class ParseConnectTest(fixtures.TestBase, AssertsCompiledSQL):
             url.make_url('mssql+pymssql://scott:tiger@somehost/test')
         connection = dialect.create_connect_args(u)
         eq_(
-            [[], {'host': 'somehost', 'password': 'tiger', 
+            [[], {'host': 'somehost', 'password': 'tiger',
                     'user': 'scott', 'database': 'test'}], connection
         )
 
@@ -1093,7 +1230,7 @@ class ParseConnectTest(fixtures.TestBase, AssertsCompiledSQL):
             url.make_url('mssql+pymssql://scott:tiger@somehost:5000/test')
         connection = dialect.create_connect_args(u)
         eq_(
-            [[], {'host': 'somehost:5000', 'password': 'tiger', 
+            [[], {'host': 'somehost:5000', 'password': 'tiger',
                     'user': 'scott', 'database': 'test'}], connection
         )
 
@@ -1762,6 +1899,13 @@ class BinaryTest(fixtures.TestBase, AssertsExecutionResults):
         fp.close()
         return stream
 
+class InfoCoerceUnicodeTest(fixtures.TestBase):
+    def test_info_unicode_coercion(self):
+        from sqlalchemy.dialects.mssql.information_schema import CoerceUnicode
+
+        dialect = mssql.dialect()
+        value = CoerceUnicode().bind_processor(dialect)('a string')
+        assert isinstance(value, unicode)
 
 class ReflectHugeViewTest(fixtures.TestBase):
     __only_on__ = 'mssql'
@@ -1778,7 +1922,7 @@ class ReflectHugeViewTest(fixtures.TestBase):
         )
         self.view_str = view_str = \
             "CREATE VIEW huge_named_view AS SELECT %s FROM base_table" % (
-            ",".join("long_named_column_number_%d" % i 
+            ",".join("long_named_column_number_%d" % i
                         for i in xrange(self.col_num))
             )
         assert len(view_str) > 4000
