@@ -440,7 +440,7 @@ class CompatFlagsTest(fixtures.TestBase, AssertsCompiledSQL):
         self.assert_compile(Unicode(50),"VARCHAR2(50)",dialect=dialect)
         self.assert_compile(UnicodeText(),"CLOB",dialect=dialect)
 
-        dialect = oracle.dialect(implicit_returning=True, 
+        dialect = oracle.dialect(implicit_returning=True,
                                     dbapi=testing.db.dialect.dbapi)
         dialect._get_server_version_info = server_version_info
         dialect.initialize(testing.db.connect())
@@ -483,23 +483,23 @@ class MultiSchemaTest(fixtures.TestBase, AssertsCompiledSQL):
 
         for stmt in """
 create table test_schema.parent(
-    id integer primary key, 
+    id integer primary key,
     data varchar2(50)
 );
 
 create table test_schema.child(
     id integer primary key,
-    data varchar2(50), 
+    data varchar2(50),
     parent_id integer references test_schema.parent(id)
 );
 
 create synonym test_schema.ptable for test_schema.parent;
 create synonym test_schema.ctable for test_schema.child;
 
--- can't make a ref from local schema to the 
--- remote schema's table without this, 
+-- can't make a ref from local schema to the
+-- remote schema's table without this,
 -- *and* cant give yourself a grant !
--- so we give it to public.  ideas welcome. 
+-- so we give it to public.  ideas welcome.
 grant references on test_schema.parent to public;
 grant references on test_schema.child to public;
 """.split(";"):
@@ -520,11 +520,11 @@ drop synonym test_schema.ptable;
     def test_create_same_names_explicit_schema(self):
         schema = testing.db.dialect.default_schema_name
         meta = MetaData(testing.db)
-        parent = Table('parent', meta, 
+        parent = Table('parent', meta,
             Column('pid', Integer, primary_key=True),
             schema=schema
         )
-        child = Table('child', meta, 
+        child = Table('child', meta,
             Column('cid', Integer, primary_key=True),
             Column('pid', Integer, ForeignKey('%s.parent.pid' % schema)),
             schema=schema
@@ -539,10 +539,10 @@ drop synonym test_schema.ptable;
 
     def test_create_same_names_implicit_schema(self):
         meta = MetaData(testing.db)
-        parent = Table('parent', meta, 
+        parent = Table('parent', meta,
             Column('pid', Integer, primary_key=True),
         )
-        child = Table('child', meta, 
+        child = Table('child', meta,
             Column('cid', Integer, primary_key=True),
             Column('pid', Integer, ForeignKey('parent.pid')),
         )
@@ -560,7 +560,7 @@ drop synonym test_schema.ptable;
         parent = Table('parent', meta, autoload=True, schema='test_schema')
         child = Table('child', meta, autoload=True, schema='test_schema')
 
-        self.assert_compile(parent.join(child), 
+        self.assert_compile(parent.join(child),
                 "test_schema.parent JOIN test_schema.child ON "
                 "test_schema.parent.id = test_schema.child.parent_id")
         select([parent, child]).\
@@ -655,13 +655,82 @@ class ConstraintTest(fixtures.TestBase):
                     onupdate='CASCADE'))
         assert_raises(exc.SAWarning, bat.create)
 
-class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
-    __only_on__ = 'oracle'
+
+class TwoPhaseTest(fixtures.TablesTest):
+    """test cx_oracle two phase, which remains in a semi-broken state
+    so requires a carefully written test."""
+
+    __only_on__ = 'oracle+cx_oracle'
+
+    @classmethod
+    def define_tables(cls, metadata):
+        Table('datatable', metadata,
+                Column('id', Integer, primary_key=True),
+                Column('data', String(50))
+            )
+
+    def _connection(self):
+        conn = testing.db.connect()
+        conn.detach()
+        return conn
+
+    def _assert_data(self, rows):
+        eq_(
+            testing.db.scalar("select count(*) from datatable"),
+            rows
+        )
+    def test_twophase_prepare_false(self):
+        conn = self._connection()
+        for i in xrange(2):
+            trans = conn.begin_twophase()
+            conn.execute("select 1 from dual")
+            trans.prepare()
+            trans.commit()
+        conn.close()
+        self._assert_data(0)
+
+    def test_twophase_prepare_true(self):
+        conn = self._connection()
+        for i in xrange(2):
+            trans = conn.begin_twophase()
+            conn.execute("insert into datatable (id, data) "
+                    "values (%s, 'somedata')" % i)
+            trans.prepare()
+            trans.commit()
+        conn.close()
+        self._assert_data(2)
+
+    def test_twophase_rollback(self):
+        conn = self._connection()
+        trans = conn.begin_twophase()
+        conn.execute("insert into datatable (id, data) "
+                "values (%s, 'somedata')" % 1)
+        trans.rollback()
+
+        trans = conn.begin_twophase()
+        conn.execute("insert into datatable (id, data) "
+                "values (%s, 'somedata')" % 1)
+        trans.prepare()
+        trans.commit()
+
+        conn.close()
+        self._assert_data(1)
+
+    def test_not_prepared(self):
+        conn = self._connection()
+        trans = conn.begin_twophase()
+        conn.execute("insert into datatable (id, data) "
+                "values (%s, 'somedata')" % 1)
+        trans.commit()
+        conn.close()
+        self._assert_data(1)
+
+class DialectTypesTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = oracle.OracleDialect()
 
     def test_no_clobs_for_string_params(self):
-        """test that simple string params get a DBAPI type of 
-        VARCHAR, not CLOB. This is to prevent setinputsizes 
+        """test that simple string params get a DBAPI type of
+        VARCHAR, not CLOB. This is to prevent setinputsizes
         from setting up cx_oracle.CLOBs on
         string-based bind params [ticket:793]."""
 
@@ -678,35 +747,8 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
         b = bindparam("foo", u"hello world!")
         assert b.type.dialect_impl(dialect).get_dbapi_type(dbapi) == 'STRING'
 
-    @testing.fails_on('+zxjdbc', 'zxjdbc lacks the FIXED_CHAR dbapi type')
-    def test_fixed_char(self):
-        m = MetaData(testing.db)
-        t = Table('t1', m, 
-            Column('id', Integer, primary_key=True),
-            Column('data', CHAR(30), nullable=False)
-        )
-
-        t.create()
-        try:
-            t.insert().execute(
-                dict(id=1, data="value 1"),
-                dict(id=2, data="value 2"),
-                dict(id=3, data="value 3")
-            )
-
-            eq_(t.select().where(t.c.data=='value 2').execute().fetchall(), 
-                [(2, 'value 2                       ')]
-                )
-
-            m2 = MetaData(testing.db)
-            t2 = Table('t1', m2, autoload=True)
-            assert type(t2.c.data.type) is CHAR
-            eq_(t2.select().where(t2.c.data=='value 2').execute().fetchall(), 
-                [(2, 'value 2                       ')]
-                )
-
-        finally:
-            t.drop()
+    def test_long(self):
+        self.assert_compile(oracle.LONG(), "LONG")
 
     def test_type_adapt(self):
         dialect = cx_oracle.dialect()
@@ -727,6 +769,80 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
             assert isinstance(start.dialect_impl(dialect), test), \
                     "wanted %r got %r" % (test, start.dialect_impl(dialect))
 
+    def test_raw_compile(self):
+        self.assert_compile(oracle.RAW(), "RAW")
+        self.assert_compile(oracle.RAW(35), "RAW(35)")
+
+    def test_char_length(self):
+        self.assert_compile(VARCHAR(50),"VARCHAR(50 CHAR)")
+
+        oracle8dialect = oracle.dialect()
+        oracle8dialect.server_version_info = (8, 0)
+        self.assert_compile(VARCHAR(50),"VARCHAR(50)",dialect=oracle8dialect)
+
+        self.assert_compile(NVARCHAR(50),"NVARCHAR2(50)")
+        self.assert_compile(CHAR(50),"CHAR(50)")
+
+    def test_varchar_types(self):
+        dialect = oracle.dialect()
+        for typ, exp in [
+            (String(50), "VARCHAR2(50 CHAR)"),
+            (Unicode(50), "NVARCHAR2(50)"),
+            (NVARCHAR(50), "NVARCHAR2(50)"),
+            (VARCHAR(50), "VARCHAR(50 CHAR)"),
+            (oracle.NVARCHAR2(50), "NVARCHAR2(50)"),
+            (oracle.VARCHAR2(50), "VARCHAR2(50 CHAR)"),
+        ]:
+            self.assert_compile(typ, exp, dialect=dialect)
+
+    def test_interval(self):
+        for type_, expected in [(oracle.INTERVAL(),
+                                'INTERVAL DAY TO SECOND'),
+                                (oracle.INTERVAL(day_precision=3),
+                                'INTERVAL DAY(3) TO SECOND'),
+                                (oracle.INTERVAL(second_precision=5),
+                                'INTERVAL DAY TO SECOND(5)'),
+                                (oracle.INTERVAL(day_precision=2,
+                                second_precision=5),
+                                'INTERVAL DAY(2) TO SECOND(5)')]:
+            self.assert_compile(type_, expected)
+
+
+class TypesTest(fixtures.TestBase):
+    __only_on__ = 'oracle'
+    __dialect__ = oracle.OracleDialect()
+
+
+    @testing.fails_on('+zxjdbc', 'zxjdbc lacks the FIXED_CHAR dbapi type')
+    def test_fixed_char(self):
+        m = MetaData(testing.db)
+        t = Table('t1', m,
+            Column('id', Integer, primary_key=True),
+            Column('data', CHAR(30), nullable=False)
+        )
+
+        t.create()
+        try:
+            t.insert().execute(
+                dict(id=1, data="value 1"),
+                dict(id=2, data="value 2"),
+                dict(id=3, data="value 3")
+            )
+
+            eq_(t.select().where(t.c.data=='value 2').execute().fetchall(),
+                [(2, 'value 2                       ')]
+                )
+
+            m2 = MetaData(testing.db)
+            t2 = Table('t1', m2, autoload=True)
+            assert type(t2.c.data.type) is CHAR
+            eq_(t2.select().where(t2.c.data=='value 2').execute().fetchall(),
+                [(2, 'value 2                       ')]
+                )
+
+        finally:
+            t.drop()
+
     @testing.requires.returning
     def test_int_not_float(self):
         m = MetaData(testing.db)
@@ -769,37 +885,25 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
     @testing.fails_on('+zxjdbc',
                       'Not yet known how to pass values of the '
                       'INTERVAL type')
+    @testing.provide_metadata
     def test_interval(self):
-        for type_, expected in [(oracle.INTERVAL(),
-                                'INTERVAL DAY TO SECOND'),
-                                (oracle.INTERVAL(day_precision=3),
-                                'INTERVAL DAY(3) TO SECOND'),
-                                (oracle.INTERVAL(second_precision=5),
-                                'INTERVAL DAY TO SECOND(5)'),
-                                (oracle.INTERVAL(day_precision=2,
-                                second_precision=5),
-                                'INTERVAL DAY(2) TO SECOND(5)')]:
-            self.assert_compile(type_, expected)
-        metadata = MetaData(testing.db)
+        metadata = self.metadata
         interval_table = Table('intervaltable', metadata, Column('id',
                                Integer, primary_key=True,
                                test_needs_autoincrement=True),
                                Column('day_interval',
                                oracle.INTERVAL(day_precision=3)))
         metadata.create_all()
-        try:
-            interval_table.insert().\
-                execute(day_interval=datetime.timedelta(days=35,
-                    seconds=5743))
-            row = interval_table.select().execute().first()
-            eq_(row['day_interval'], datetime.timedelta(days=35,
+        interval_table.insert().\
+            execute(day_interval=datetime.timedelta(days=35,
                 seconds=5743))
-        finally:
-            metadata.drop_all()
+        row = interval_table.select().execute().first()
+        eq_(row['day_interval'], datetime.timedelta(days=35,
+            seconds=5743))
 
     def test_numerics(self):
         m = MetaData(testing.db)
-        t1 = Table('t1', m, 
+        t1 = Table('t1', m,
             Column('intcol', Integer),
             Column('numericcol', Numeric(precision=9, scale=2)),
             Column('floatcol1', Float()),
@@ -813,11 +917,11 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
         t1.create()
         try:
             t1.insert().execute(
-                intcol=1, 
-                numericcol=5.2, 
-                floatcol1=6.5, 
+                intcol=1,
+                numericcol=5.2,
+                floatcol1=6.5,
                 floatcol2 = 8.5,
-                doubleprec = 9.5, 
+                doubleprec = 9.5,
                 numbercol1=12,
                 numbercol2=14.85,
                 numbercol3=15.76
@@ -828,7 +932,7 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
 
             for row in (
                 t1.select().execute().first(),
-                t2.select().execute().first() 
+                t2.select().execute().first()
             ):
                 for i, (val, type_) in enumerate((
                     (1, int),
@@ -847,6 +951,15 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
         finally:
             t1.drop()
 
+
+    def test_numeric_no_decimal_mode(self):
+        engine = testing_engine(options=dict(coerce_to_decimal=False))
+        value = engine.scalar("SELECT 5.66 FROM DUAL")
+        assert isinstance(value, float)
+
+        value = testing.db.scalar("SELECT 5.66 FROM DUAL")
+        assert isinstance(value, decimal.Decimal)
+
     @testing.provide_metadata
     def test_numerics_broken_inspection(self):
         """Numeric scenarios where Oracle type info is 'broken',
@@ -869,12 +982,13 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
         foo.create()
 
         foo.insert().execute(
-            {'idata':5, 'ndata':decimal.Decimal("45.6"), 'ndata2':decimal.Decimal("45.0"), 
+            {'idata':5, 'ndata':decimal.Decimal("45.6"),
+                    'ndata2':decimal.Decimal("45.0"),
                     'nidata':decimal.Decimal('53'), 'fdata':45.68392},
         )
 
         stmt = """
-        SELECT 
+        SELECT
             idata,
             ndata,
             ndata2,
@@ -887,11 +1001,11 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
         row = testing.db.execute(stmt).fetchall()[0]
         eq_([type(x) for x in row], [int, decimal.Decimal, decimal.Decimal, int, float])
         eq_(
-            row, 
+            row,
             (5, decimal.Decimal('45.6'), decimal.Decimal('45'), 53, 45.683920000000001)
         )
 
-        # with a nested subquery, 
+        # with a nested subquery,
         # both Numeric values that don't have decimal places, regardless
         # of their originating type, come back as ints with no useful
         # typing information beyond "numeric".  So native handler
@@ -900,7 +1014,7 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
         # totally sucks.
 
         stmt = """
-        SELECT 
+        SELECT
             (SELECT (SELECT idata FROM foo) FROM DUAL) AS idata,
             (SELECT CAST((SELECT ndata FROM foo) AS NUMERIC(20, 2)) FROM DUAL)
              AS ndata,
@@ -914,25 +1028,25 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
         row = testing.db.execute(stmt).fetchall()[0]
         eq_([type(x) for x in row], [int, decimal.Decimal, int, int, decimal.Decimal])
         eq_(
-            row, 
+            row,
             (5, decimal.Decimal('45.6'), 45, 53, decimal.Decimal('45.68392'))
         )
 
-        row = testing.db.execute(text(stmt, 
+        row = testing.db.execute(text(stmt,
                                 typemap={
-                                        'idata':Integer(), 
-                                        'ndata':Numeric(20, 2), 
-                                        'ndata2':Numeric(20, 2), 
+                                        'idata':Integer(),
+                                        'ndata':Numeric(20, 2),
+                                        'ndata2':Numeric(20, 2),
                                         'nidata':Numeric(5, 0),
                                         'fdata':Float()
                                 })).fetchall()[0]
         eq_([type(x) for x in row], [int, decimal.Decimal, decimal.Decimal, decimal.Decimal, float])
-        eq_(row, 
+        eq_(row,
             (5, decimal.Decimal('45.6'), decimal.Decimal('45'), decimal.Decimal('53'), 45.683920000000001)
         )
 
         stmt = """
-        SELECT 
+        SELECT
                 anon_1.idata AS anon_1_idata,
                 anon_1.ndata AS anon_1_ndata,
                 anon_1.ndata2 AS anon_1_ndata2,
@@ -940,15 +1054,15 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
                 anon_1.fdata AS anon_1_fdata
         FROM (SELECT idata, ndata, ndata2, nidata, fdata
         FROM (
-            SELECT 
+            SELECT
                 (SELECT (SELECT idata FROM foo) FROM DUAL) AS idata,
-                (SELECT CAST((SELECT ndata FROM foo) AS NUMERIC(20, 2)) 
+                (SELECT CAST((SELECT ndata FROM foo) AS NUMERIC(20, 2))
                 FROM DUAL) AS ndata,
-                (SELECT CAST((SELECT ndata2 FROM foo) AS NUMERIC(20, 2)) 
+                (SELECT CAST((SELECT ndata2 FROM foo) AS NUMERIC(20, 2))
                 FROM DUAL) AS ndata2,
-                (SELECT CAST((SELECT nidata FROM foo) AS NUMERIC(5, 0)) 
+                (SELECT CAST((SELECT nidata FROM foo) AS NUMERIC(5, 0))
                 FROM DUAL) AS nidata,
-                (SELECT CAST((SELECT fdata FROM foo) AS FLOAT) FROM DUAL) 
+                (SELECT CAST((SELECT fdata FROM foo) AS FLOAT) FROM DUAL)
                 AS fdata
             FROM dual
         )
@@ -958,29 +1072,29 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
         eq_([type(x) for x in row], [int, decimal.Decimal, int, int, decimal.Decimal])
         eq_(row, (5, decimal.Decimal('45.6'), 45, 53, decimal.Decimal('45.68392')))
 
-        row = testing.db.execute(text(stmt, 
+        row = testing.db.execute(text(stmt,
                                 typemap={
-                                        'anon_1_idata':Integer(), 
-                                        'anon_1_ndata':Numeric(20, 2), 
-                                        'anon_1_ndata2':Numeric(20, 2), 
-                                        'anon_1_nidata':Numeric(5, 0), 
+                                        'anon_1_idata':Integer(),
+                                        'anon_1_ndata':Numeric(20, 2),
+                                        'anon_1_ndata2':Numeric(20, 2),
+                                        'anon_1_nidata':Numeric(5, 0),
                                         'anon_1_fdata':Float()
                                 })).fetchall()[0]
         eq_([type(x) for x in row], [int, decimal.Decimal, decimal.Decimal, decimal.Decimal, float])
-        eq_(row, 
+        eq_(row,
             (5, decimal.Decimal('45.6'), decimal.Decimal('45'), decimal.Decimal('53'), 45.683920000000001)
         )
 
-        row = testing.db.execute(text(stmt, 
+        row = testing.db.execute(text(stmt,
                                 typemap={
-                                        'anon_1_idata':Integer(), 
-                                        'anon_1_ndata':Numeric(20, 2, asdecimal=False), 
-                                        'anon_1_ndata2':Numeric(20, 2, asdecimal=False), 
-                                        'anon_1_nidata':Numeric(5, 0, asdecimal=False), 
+                                        'anon_1_idata':Integer(),
+                                        'anon_1_ndata':Numeric(20, 2, asdecimal=False),
+                                        'anon_1_ndata2':Numeric(20, 2, asdecimal=False),
+                                        'anon_1_nidata':Numeric(5, 0, asdecimal=False),
                                         'anon_1_fdata':Float(asdecimal=True)
                                 })).fetchall()[0]
         eq_([type(x) for x in row], [int, float, float, float, decimal.Decimal])
-        eq_(row, 
+        eq_(row,
             (5, 45.6, 45, 53, decimal.Decimal('45.68392'))
         )
 
@@ -1019,10 +1133,6 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
         for row in types_table.select().execute().fetchall():
             [row[k] for k in row.keys()]
 
-    def test_raw_compile(self):
-        self.assert_compile(oracle.RAW(), "RAW")
-        self.assert_compile(oracle.RAW(35), "RAW(35)")
-
     @testing.provide_metadata
     def test_raw_roundtrip(self):
         metadata = self.metadata
@@ -1052,7 +1162,7 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
             # nvarchar returns unicode natively.  cx_oracle
             # _OracleNVarChar type should be at play here.
             assert isinstance(
-                t2.c.data.type.dialect_impl(testing.db.dialect), 
+                t2.c.data.type.dialect_impl(testing.db.dialect),
                 cx_oracle._OracleNVarChar)
 
         data = u'm’a réveillé.'
@@ -1061,16 +1171,8 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
         eq_(res, data)
         assert isinstance(res, unicode)
 
+
     def test_char_length(self):
-        self.assert_compile(VARCHAR(50),"VARCHAR(50 CHAR)")
-
-        oracle8dialect = oracle.dialect()
-        oracle8dialect.server_version_info = (8, 0)
-        self.assert_compile(VARCHAR(50),"VARCHAR(50)",dialect=oracle8dialect)
-
-        self.assert_compile(NVARCHAR(50),"NVARCHAR2(50)")
-        self.assert_compile(CHAR(50),"CHAR(50)")
-
         metadata = MetaData(testing.db)
         t1 = Table('t1', metadata,
               Column("c1", VARCHAR(50)),
@@ -1087,17 +1189,21 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
         finally:
             t1.drop()
 
-    def test_varchar_types(self):
-        dialect = oracle.dialect()
-        for typ, exp in [
-            (String(50), "VARCHAR2(50 CHAR)"),
-            (Unicode(50), "NVARCHAR2(50)"),
-            (NVARCHAR(50), "NVARCHAR2(50)"),
-            (VARCHAR(50), "VARCHAR(50 CHAR)"),
-            (oracle.NVARCHAR2(50), "NVARCHAR2(50)"),
-            (oracle.VARCHAR2(50), "VARCHAR2(50 CHAR)"),
-        ]:
-            self.assert_compile(typ, exp, dialect=dialect)
+    @testing.provide_metadata
+    def test_long_type(self):
+        metadata = self.metadata
+
+        t = Table('t', metadata,
+                Column('data', oracle.LONG)
+            )
+        metadata.create_all(testing.db)
+        testing.db.execute(t.insert(), data='xyz')
+        eq_(
+            testing.db.scalar(select([t.c.data])),
+            "xyz"
+        )
+
+
 
     def test_longstring(self):
         metadata = MetaData(testing.db)
@@ -1119,12 +1225,12 @@ class TypesTest(fixtures.TestBase, AssertsCompiledSQL):
     def test_lobs_without_convert(self):
         engine = testing_engine(options=dict(auto_convert_lobs=False))
         metadata = MetaData()
-        t = Table("z_test", metadata, Column('id', Integer, primary_key=True), 
+        t = Table("z_test", metadata, Column('id', Integer, primary_key=True),
                  Column('data', Text), Column('bindata', LargeBinary))
         t.create(engine)
         try:
-            engine.execute(t.insert(), id=1, 
-                                        data='this is text', 
+            engine.execute(t.insert(), id=1,
+                                        data='this is text',
                                         bindata='this is binary')
             row = engine.execute(t.select()).first()
             eq_(row['data'].read(), 'this is text')
@@ -1171,17 +1277,17 @@ class DontReflectIOTTest(fixtures.TestBase):
     """test that index overflow tables aren't included in
     table_names."""
 
-    __only_on__ = 'oracle' 
+    __only_on__ = 'oracle'
 
     def setup(self):
         testing.db.execute("""
         CREATE TABLE admin_docindex(
-                token char(20), 
+                token char(20),
                 doc_id NUMBER,
                 token_frequency NUMBER,
                 token_offsets VARCHAR2(2000),
                 CONSTRAINT pk_admin_docindex PRIMARY KEY (token, doc_id))
-            ORGANIZATION INDEX 
+            ORGANIZATION INDEX
             TABLESPACE users
             PCTTHRESHOLD 20
             OVERFLOW TABLESPACE users
@@ -1205,13 +1311,13 @@ class BufferedColumnTest(fixtures.TestBase, AssertsCompiledSQL):
     def setup_class(cls):
         global binary_table, stream, meta
         meta = MetaData(testing.db)
-        binary_table = Table('binary_table', meta, 
+        binary_table = Table('binary_table', meta,
            Column('id', Integer, primary_key=True),
            Column('data', LargeBinary)
         )
         meta.create_all()
         stream = os.path.join(
-                        os.path.dirname(__file__), "..", 
+                        os.path.dirname(__file__), "..",
                         'binary_data_one.dat')
         stream = file(stream).read(12000)
 
@@ -1238,7 +1344,7 @@ class UnsupportedIndexReflectTest(fixtures.TestBase):
     def setup(self):
         global metadata
         metadata = MetaData(testing.db)
-        t1 = Table('test_index_reflect', metadata, 
+        t1 = Table('test_index_reflect', metadata,
                     Column('data', String(20), primary_key=True)
                 )
         metadata.create_all()
@@ -1246,6 +1352,7 @@ class UnsupportedIndexReflectTest(fixtures.TestBase):
     def teardown(self):
         metadata.drop_all()
 
+    @testing.emits_warning("No column names")
     def test_reflect_functional_index(self):
         testing.db.execute('CREATE INDEX DATA_IDX ON '
                            'TEST_INDEX_REFLECT (UPPER(DATA))')
@@ -1268,7 +1375,7 @@ class RoundTripIndexTest(fixtures.TestBase):
         )
 
         # "group" is a keyword, so lower case
-        normalind = Index('tableind', table.c.id_b, table.c.group) 
+        normalind = Index('tableind', table.c.id_b, table.c.group)
 
         # create
         metadata.create_all()
