@@ -1,4 +1,4 @@
-from sqlalchemy import Integer, ForeignKey
+from sqlalchemy import Integer, ForeignKey, String
 from sqlalchemy.types import PickleType, TypeDecorator, VARCHAR
 from sqlalchemy.orm import mapper, Session, composite
 from sqlalchemy.orm.mapper import Mapper
@@ -17,6 +17,17 @@ class Foo(fixtures.BasicEntity):
 class SubFoo(Foo):
     pass
 
+class FooWithEq(object):
+    def __init__(self, **kw):
+        for k in kw:
+            setattr(self, k, kw[k])
+
+    def __hash__(self):
+        return hash(self.id)
+
+    def __eq__(self, other):
+        return self.id == other.id
+
 class _MutableDictTestBase(object):
     run_define_tables = 'each'
 
@@ -115,6 +126,17 @@ class _MutableDictTestBase(object):
             f2.data['a'] = 'c'
             assert f2 in sess.dirty
 
+    def test_unrelated_flush(self):
+        sess = Session()
+        f1 = Foo(data={"a":"b"}, unrelated_data="unrelated")
+        sess.add(f1)
+        sess.flush()
+        f1.unrelated_data = "unrelated 2"
+        sess.flush()
+        f1.data["a"] = "c"
+        sess.commit()
+        eq_(f1.data["a"], "c")
+
     def _test_non_mutable(self):
         sess = Session()
 
@@ -132,10 +154,13 @@ class MutableWithScalarPickleTest(_MutableDictTestBase, fixtures.MappedTest):
     def define_tables(cls, metadata):
         MutationDict = cls._type_fixture()
 
+        mutable_pickle = MutationDict.as_mutable(PickleType)
         Table('foo', metadata,
             Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
-            Column('data', MutationDict.as_mutable(PickleType)),
-            Column('non_mutable_data', PickleType)
+            Column('skip', mutable_pickle),
+            Column('data', mutable_pickle),
+            Column('non_mutable_data', PickleType),
+            Column('unrelated_data', String(50))
         )
 
     def test_non_mutable(self):
@@ -168,7 +193,8 @@ class MutableWithScalarJSONTest(_MutableDictTestBase, fixtures.MappedTest):
         Table('foo', metadata,
             Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('data', MutationDict.as_mutable(JSONEncodedDict)),
-            Column('non_mutable_data', JSONEncodedDict)
+            Column('non_mutable_data', JSONEncodedDict),
+            Column('unrelated_data', String(50))
         )
 
     def test_non_mutable(self):
@@ -182,7 +208,8 @@ class MutableAssocWithAttrInheritTest(_MutableDictTestBase, fixtures.MappedTest)
         Table('foo', metadata,
             Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('data', PickleType),
-            Column('non_mutable_data', PickleType)
+            Column('non_mutable_data', PickleType),
+            Column('unrelated_data', String(50))
         )
 
         Table('subfoo', metadata,
@@ -227,7 +254,9 @@ class MutableAssociationScalarPickleTest(_MutableDictTestBase, fixtures.MappedTe
 
         Table('foo', metadata,
             Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
-            Column('data', PickleType)
+            Column('skip', PickleType),
+            Column('data', PickleType),
+            Column('unrelated_data', String(50))
         )
 
 class MutableAssociationScalarJSONTest(_MutableDictTestBase, fixtures.MappedTest):
@@ -257,7 +286,8 @@ class MutableAssociationScalarJSONTest(_MutableDictTestBase, fixtures.MappedTest
 
         Table('foo', metadata,
             Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
-            Column('data', JSONEncodedDict)
+            Column('data', JSONEncodedDict),
+            Column('unrelated_data', String(50))
         )
 
 
@@ -267,9 +297,16 @@ class _CompositeTestBase(object):
         Table('foo', metadata,
             Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('x', Integer),
-            Column('y', Integer)
+            Column('y', Integer),
+            Column('unrelated_data', String(50))
         )
 
+    def setup(self):
+        from sqlalchemy.ext import mutable
+        mutable._setup_composite_listener()
+        super(_CompositeTestBase, self).setup()
+
+
     def teardown(self):
         # clear out mapper events
         Mapper.dispatch._clear()
@@ -297,12 +334,10 @@ class _CompositeTestBase(object):
                 return self.x, self.y
 
             def __getstate__(self):
-                d = dict(self.__dict__)
-                d.pop('_parents', None)
-                return d
+                return self.x, self.y
 
-            #def __setstate__(self, state):
-            #    self.x, self.y = state
+            def __setstate__(self, state):
+                self.x, self.y = state
 
             def __eq__(self, other):
                 return isinstance(other, Point) and \
@@ -310,6 +345,23 @@ class _CompositeTestBase(object):
                     other.y == self.y
         return Point
 
+class MutableCompositesUnpickleTest(_CompositeTestBase, fixtures.MappedTest):
+
+    @classmethod
+    def setup_mappers(cls):
+        foo = cls.tables.foo
+
+        cls.Point = cls._type_fixture()
+
+        mapper(FooWithEq, foo, properties={
+            'data':composite(cls.Point, foo.c.x, foo.c.y)
+        })
+
+    def test_unpickle_modified_eq(self):
+        u1 = FooWithEq(data=self.Point(3, 5))
+        for loads, dumps in picklers():
+            loads(dumps(u1))
+
 class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
 
     @classmethod
@@ -352,6 +404,101 @@ class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
             f2.data.y = 12
             assert f2 in sess.dirty
 
+    def test_set_none(self):
+        sess = Session()
+        f1 = Foo(data=None)
+        sess.add(f1)
+        sess.commit()
+        eq_(f1.data, Point(None, None))
+
+        f1.data.y = 5
+        sess.commit()
+        eq_(f1.data, Point(None, 5))
+
+    def test_set_illegal(self):
+        f1 = Foo()
+        assert_raises_message(
+            ValueError,
+            "Attribute 'data' does not accept objects",
+            setattr, f1, 'data', 'foo'
+        )
+
+    def test_unrelated_flush(self):
+        sess = Session()
+        f1 = Foo(data=Point(3, 4), unrelated_data="unrelated")
+        sess.add(f1)
+        sess.flush()
+        f1.unrelated_data = "unrelated 2"
+        sess.flush()
+        f1.data.x = 5
+        sess.commit()
+
+        eq_(f1.data.x, 5)
+
+class MutableCompositeCustomCoerceTest(_CompositeTestBase, fixtures.MappedTest):
+    @classmethod
+    def _type_fixture(cls):
+
+        from sqlalchemy.ext.mutable import MutableComposite
+
+        global Point
+
+        class Point(MutableComposite):
+            def __init__(self, x, y):
+                self.x = x
+                self.y = y
+
+            @classmethod
+            def coerce(cls, key, value):
+                if isinstance(value, tuple):
+                    value = Point(*value)
+                return value
+
+            def __setattr__(self, key, value):
+                object.__setattr__(self, key, value)
+                self.changed()
+
+            def __composite_values__(self):
+                return self.x, self.y
+
+            def __getstate__(self):
+                return self.x, self.y
+
+            def __setstate__(self, state):
+                self.x, self.y = state
+
+            def __eq__(self, other):
+                return isinstance(other, Point) and \
+                    other.x == self.x and \
+                    other.y == self.y
+        return Point
+
+
+    @classmethod
+    def setup_mappers(cls):
+        foo = cls.tables.foo
+
+        Point = cls._type_fixture()
+
+        mapper(Foo, foo, properties={
+            'data': composite(Point, foo.c.x, foo.c.y)
+        })
+
+    def test_custom_coerce(self):
+        f = Foo()
+        f.data = (3, 4)
+        eq_(f.data, Point(3, 4))
+
+    def test_round_trip_ok(self):
+        sess = Session()
+        f = Foo()
+        f.data = (3, 4)
+
+        sess.add(f)
+        sess.commit()
+
+        eq_(f.data, Point(3, 4))
+
 
 class MutableInheritedCompositesTest(_CompositeTestBase, fixtures.MappedTest):
     @classmethod
