@@ -16,6 +16,151 @@ MyTest = None
 MyTest2 = None
 
 
+class AttributeImplAPITest(fixtures.MappedTest):
+    def _scalar_obj_fixture(self):
+        class A(object):
+            pass
+        class B(object):
+            pass
+        instrumentation.register_class(A)
+        instrumentation.register_class(B)
+        attributes.register_attribute(A, "b", uselist=False, useobject=True)
+        return A, B
+
+    def _collection_obj_fixture(self):
+        class A(object):
+            pass
+        class B(object):
+            pass
+        instrumentation.register_class(A)
+        instrumentation.register_class(B)
+        attributes.register_attribute(A, "b", uselist=True, useobject=True)
+        return A, B
+
+    def test_scalar_obj_remove_invalid(self):
+        A, B = self._scalar_obj_fixture()
+
+        a1 = A()
+        b1 = B()
+        b2 = B()
+
+        A.b.impl.append(
+            attributes.instance_state(a1),
+            attributes.instance_dict(a1), b1, None
+        )
+
+        assert a1.b is b1
+
+        assert_raises_message(
+            ValueError,
+            "Object <B at .*?> not "
+            "associated with <A at .*?> on attribute 'b'",
+            A.b.impl.remove,
+                attributes.instance_state(a1),
+                attributes.instance_dict(a1), b2, None
+        )
+
+    def test_scalar_obj_pop_invalid(self):
+        A, B = self._scalar_obj_fixture()
+
+        a1 = A()
+        b1 = B()
+        b2 = B()
+
+        A.b.impl.append(
+            attributes.instance_state(a1),
+            attributes.instance_dict(a1), b1, None
+        )
+
+        assert a1.b is b1
+
+        A.b.impl.pop(
+            attributes.instance_state(a1),
+            attributes.instance_dict(a1), b2, None
+        )
+        assert a1.b is b1
+
+    def test_scalar_obj_pop_valid(self):
+        A, B = self._scalar_obj_fixture()
+
+        a1 = A()
+        b1 = B()
+
+        A.b.impl.append(
+            attributes.instance_state(a1),
+            attributes.instance_dict(a1), b1, None
+        )
+
+        assert a1.b is b1
+
+        A.b.impl.pop(
+            attributes.instance_state(a1),
+            attributes.instance_dict(a1), b1, None
+        )
+        assert a1.b is None
+
+    def test_collection_obj_remove_invalid(self):
+        A, B = self._collection_obj_fixture()
+
+        a1 = A()
+        b1 = B()
+        b2 = B()
+
+        A.b.impl.append(
+            attributes.instance_state(a1),
+            attributes.instance_dict(a1), b1, None
+        )
+
+        assert a1.b == [b1]
+
+        assert_raises_message(
+            ValueError,
+            r"list.remove\(x\): x not in list",
+            A.b.impl.remove,
+                attributes.instance_state(a1),
+                attributes.instance_dict(a1), b2, None
+        )
+
+    def test_collection_obj_pop_invalid(self):
+        A, B = self._collection_obj_fixture()
+
+        a1 = A()
+        b1 = B()
+        b2 = B()
+
+        A.b.impl.append(
+            attributes.instance_state(a1),
+            attributes.instance_dict(a1), b1, None
+        )
+
+        assert a1.b == [b1]
+
+        A.b.impl.pop(
+            attributes.instance_state(a1),
+            attributes.instance_dict(a1), b2, None
+        )
+        assert a1.b == [b1]
+
+    def test_collection_obj_pop_valid(self):
+        A, B = self._collection_obj_fixture()
+
+        a1 = A()
+        b1 = B()
+
+        A.b.impl.append(
+            attributes.instance_state(a1),
+            attributes.instance_dict(a1), b1, None
+        )
+
+        assert a1.b == [b1]
+
+        A.b.impl.pop(
+            attributes.instance_state(a1),
+            attributes.instance_dict(a1), b1, None
+        )
+        assert a1.b == []
+
+
 class AttributesTest(fixtures.ORMTest):
     def setup(self):
         global MyTest, MyTest2
@@ -154,9 +299,9 @@ class AttributesTest(fixtures.ORMTest):
 
         instrumentation.register_class(Foo)
         instrumentation.register_class(Bar)
-        attributes.register_attribute(Foo, 
-                                    'bars', 
-                                    uselist=True, 
+        attributes.register_attribute(Foo,
+                                    'bars',
+                                    uselist=True,
                                     useobject=True)
 
         assert_raises_message(
@@ -321,20 +466,20 @@ class AttributesTest(fixtures.ORMTest):
                 return attributes.PASSIVE_NO_RESULT
             return b2
 
-        attributes.register_attribute(Foo, 'bars', 
-                               uselist=True, 
-                               useobject=True, 
+        attributes.register_attribute(Foo, 'bars',
+                               uselist=True,
+                               useobject=True,
                                callable_=loadcollection,
                                extension=[ReceiveEvents('bars')])
 
-        attributes.register_attribute(Foo, 'bar', 
-                              uselist=False, 
-                              useobject=True, 
+        attributes.register_attribute(Foo, 'bar',
+                              uselist=False,
+                              useobject=True,
                               callable_=loadscalar,
                               extension=[ReceiveEvents('bar')])
 
-        attributes.register_attribute(Foo, 'scalar', 
-                            uselist=False, 
+        attributes.register_attribute(Foo, 'scalar',
+                            uselist=False,
                             useobject=False, extension=[ReceiveEvents('scalar')])
 
 
@@ -464,9 +609,8 @@ class AttributesTest(fixtures.ORMTest):
             ('remove', f, [4,5,6])
         ])
 
-
     def test_lazytrackparent(self):
-        """test that the "hasparent" flag works properly 
+        """test that the "hasparent" flag works properly
            when lazy loaders and backrefs are used
 
         """
@@ -487,9 +631,9 @@ class AttributesTest(fixtures.ORMTest):
         # create objects as if they'd been freshly loaded from the database (without history)
         b = Blog()
         p1 = Post()
-        attributes.instance_state(b).set_callable(attributes.instance_dict(b), 
+        attributes.instance_state(b).set_callable(attributes.instance_dict(b),
                                                     'posts', lambda passive:[p1])
-        attributes.instance_state(p1).set_callable(attributes.instance_dict(p1), 
+        attributes.instance_state(p1).set_callable(attributes.instance_dict(p1),
                                                     'blog', lambda passive:b)
         p1, attributes.instance_state(b).commit_all(attributes.instance_dict(b))
 
@@ -508,6 +652,25 @@ class AttributesTest(fixtures.ORMTest):
         assert attributes.has_parent(Blog, p2, 'posts')
         assert attributes.has_parent(Post, b2, 'blog')
 
+    def test_illegal_trackparent(self):
+        class Post(object):pass
+        class Blog(object):pass
+        instrumentation.register_class(Post)
+        instrumentation.register_class(Blog)
+
+        attributes.register_attribute(Post, 'blog', useobject=True)
+        assert_raises_message(
+            AssertionError,
+            "This AttributeImpl is not configured to track parents.",
+            attributes.has_parent, Post, Blog(), 'blog'
+        )
+        assert_raises_message(
+            AssertionError,
+            "This AttributeImpl is not configured to track parents.",
+            Post.blog.impl.sethasparent, "x", "x", True
+        )
+
+
     def test_inheritance(self):
         """tests that attributes are polymorphic"""
 
@@ -524,11 +687,11 @@ class AttributesTest(fixtures.ORMTest):
             return "this is the bar attr"
         def func3(state, passive):
             return "this is the shared attr"
-        attributes.register_attribute(Foo, 'element', uselist=False, 
+        attributes.register_attribute(Foo, 'element', uselist=False,
                                             callable_=func1, useobject=True)
-        attributes.register_attribute(Foo, 'element2', uselist=False, 
+        attributes.register_attribute(Foo, 'element2', uselist=False,
                                             callable_=func3, useobject=True)
-        attributes.register_attribute(Bar, 'element', uselist=False, 
+        attributes.register_attribute(Bar, 'element', uselist=False,
                                             callable_=func2, useobject=True)
 
         x = Foo()
@@ -768,6 +931,81 @@ class AttributesTest(fixtures.ORMTest):
         except sa_exc.ArgumentError, e:
             assert False
 
+class GetNoValueTest(fixtures.ORMTest):
+    def _fixture(self, expected):
+        class Foo(object):
+            pass
+
+        class Bar(object):
+            pass
+
+        def lazy_callable(state, passive):
+            return expected
+
+        instrumentation.register_class(Foo)
+        instrumentation.register_class(Bar)
+        if expected is not None:
+            attributes.register_attribute(Foo,
+                        "attr", useobject=True,
+                        uselist=False, callable_=lazy_callable)
+        else:
+            attributes.register_attribute(Foo,
+                        "attr", useobject=True,
+                        uselist=False)
+
+        f1 = Foo()
+        return Foo.attr.impl,\
+                attributes.instance_state(f1), \
+                attributes.instance_dict(f1)
+
+
+    def test_passive_no_result(self):
+        attr, state, dict_ = self._fixture(attributes.PASSIVE_NO_RESULT)
+        eq_(
+            attr.get(state, dict_, passive=attributes.PASSIVE_NO_INITIALIZE),
+            attributes.PASSIVE_NO_RESULT
+        )
+
+    def test_passive_no_result_never_set(self):
+        attr, state, dict_ = self._fixture(attributes.NEVER_SET)
+        eq_(
+            attr.get(state, dict_, passive=attributes.PASSIVE_NO_INITIALIZE),
+            attributes.PASSIVE_NO_RESULT
+        )
+        assert 'attr' not in dict_
+
+    def test_passive_ret_never_set_never_set(self):
+        attr, state, dict_ = self._fixture(attributes.NEVER_SET)
+        eq_(
+            attr.get(state, dict_, passive=attributes.PASSIVE_RETURN_NEVER_SET),
+            attributes.NEVER_SET
+        )
+        assert 'attr' not in dict_
+
+    def test_passive_ret_never_set_empty(self):
+        attr, state, dict_ = self._fixture(None)
+        eq_(
+            attr.get(state, dict_, passive=attributes.PASSIVE_RETURN_NEVER_SET),
+            attributes.NEVER_SET
+        )
+        assert 'attr' not in dict_
+
+    def test_passive_no_result_empty(self):
+        attr, state, dict_ = self._fixture(None)
+        eq_(
+            attr.get(state, dict_, passive=attributes.PASSIVE_NO_RESULT),
+            None
+        )
+        assert 'attr' in dict_
+
+    def test_off_empty(self):
+        attr, state, dict_ = self._fixture(None)
+        eq_(
+            attr.get(state, dict_, passive=attributes.PASSIVE_OFF),
+            None
+        )
+        assert 'attr' in dict_
+
 class UtilTest(fixtures.ORMTest):
     def test_helpers(self):
         class Foo(object):
@@ -864,7 +1102,7 @@ class BackrefTest(fixtures.ORMTest):
         instrumentation.register_class(Port)
         instrumentation.register_class(Jack)
 
-        attributes.register_attribute(Port, 'jack', uselist=False, 
+        attributes.register_attribute(Port, 'jack', uselist=False,
                                             useobject=True, backref="port")
 
         attributes.register_attribute(Jack, 'port', uselist=False,
@@ -968,6 +1206,72 @@ class BackrefTest(fixtures.ORMTest):
         # and this condition changes.
         assert c1 in p1.children
 
+class CyclicBackrefAssertionTest(fixtures.TestBase):
+    """test that infinite recursion due to incorrect backref assignments
+    is blocked.
+
+    """
+    def test_scalar_set_type_assertion(self):
+        A, B, C = self._scalar_fixture()
+        c1 = C()
+        b1 = B()
+        assert_raises_message(
+            ValueError,
+            "Object <B at .*> not associated with attribute of type C.a",
+            setattr, c1, 'a', b1
+        )
+
+    def test_collection_append_type_assertion(self):
+        A, B, C = self._collection_fixture()
+        c1 = C()
+        b1 = B()
+        assert_raises_message(
+            ValueError,
+            "Object <B at .*> not associated with attribute of type C.a",
+            c1.a.append, b1
+        )
+
+    def _scalar_fixture(self):
+        class A(object):
+            pass
+        class B(object):
+            pass
+        class C(object):
+            pass
+        instrumentation.register_class(A)
+        instrumentation.register_class(B)
+        instrumentation.register_class(C)
+        attributes.register_attribute(C, 'a', backref='c', useobject=True)
+        attributes.register_attribute(C, 'b', backref='c', useobject=True)
+
+        attributes.register_attribute(A, 'c', backref='a', useobject=True,
+                        uselist=True)
+        attributes.register_attribute(B, 'c', backref='b', useobject=True,
+                        uselist=True)
+
+        return A, B, C
+
+    def _collection_fixture(self):
+        class A(object):
+            pass
+        class B(object):
+            pass
+        class C(object):
+            pass
+        instrumentation.register_class(A)
+        instrumentation.register_class(B)
+        instrumentation.register_class(C)
+
+        attributes.register_attribute(C, 'a', backref='c', useobject=True,
+                                                uselist=True)
+        attributes.register_attribute(C, 'b', backref='c', useobject=True,
+                                                uselist=True)
+
+        attributes.register_attribute(A, 'c', backref='a', useobject=True)
+        attributes.register_attribute(B, 'c', backref='b', useobject=True)
+
+        return A, B, C
+
 class PendingBackrefTest(fixtures.ORMTest):
     def setup(self):
         global Post, Blog, called, lazy_load
@@ -1037,7 +1341,7 @@ class PendingBackrefTest(fixtures.ORMTest):
         p4.blog = b
         assert called[0] == 0
         eq_(attributes.instance_state(b).
-                get_history('posts', attributes.PASSIVE_OFF), 
+                get_history('posts', attributes.PASSIVE_OFF),
                             ([p, p4], [p1, p2, p3], []))
         assert called[0] == 1
 
@@ -1060,7 +1364,7 @@ class PendingBackrefTest(fixtures.ORMTest):
         lazy_load = [p, p2]
         # lazy loaded + pending get added together.
         # This isn't seen often with the ORM due
-        # to usual practices surrounding the 
+        # to usual practices surrounding the
         # load/flush/load cycle.
         eq_(b.posts, [p, p2, p])
         eq_(called[0], 1)
@@ -1105,7 +1409,7 @@ class HistoryTest(fixtures.TestBase):
 
         instrumentation.register_class(Foo)
         attributes.register_attribute(
-                    Foo, 'someattr', 
+                    Foo, 'someattr',
                     uselist=uselist,
                     useobject=useobject,
                     active_history=active_history,
@@ -1125,10 +1429,10 @@ class HistoryTest(fixtures.TestBase):
                 useobject=True)
         return Foo, Bar
 
-    def _someattr_history(self, f):
+    def _someattr_history(self, f, **kw):
         return attributes.get_state_history(
                     attributes.instance_state(f),
-                    'someattr')
+                    'someattr', **kw)
 
     def _commit_someattr(self, f):
         attributes.instance_state(f).commit(attributes.instance_dict(f),
@@ -1141,20 +1445,20 @@ class HistoryTest(fixtures.TestBase):
             attributes.instance_dict(f))
 
     def test_committed_value_init(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         eq_(self._someattr_committed_state(f), None)
 
     def test_committed_value_set(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.someattr = 3
         eq_(self._someattr_committed_state(f), None)
 
     def test_committed_value_set_commit(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.someattr = 3
@@ -1162,13 +1466,25 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_committed_state(f), 3)
 
     def test_scalar_init(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         eq_(self._someattr_history(f), ((), (), ()))
 
+    def test_object_init(self):
+        Foo = self._fixture(uselist=False, useobject=True,
+                                active_history=False)
+        f = Foo()
+        eq_(self._someattr_history(f), ((), (), ()))
+
+    def test_object_init_active_history(self):
+        Foo = self._fixture(uselist=False, useobject=True,
+                                active_history=True)
+        f = Foo()
+        eq_(self._someattr_history(f), ((), (), ()))
+
     def test_scalar_no_init_side_effect(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         self._someattr_history(f)
@@ -1177,14 +1493,14 @@ class HistoryTest(fixtures.TestBase):
         assert 'someattr' not in attributes.instance_state(f).committed_state
 
     def test_scalar_set(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.someattr = 'hi'
         eq_(self._someattr_history(f), (['hi'], (), ()))
 
     def test_scalar_set_commit(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.someattr = 'hi'
@@ -1192,7 +1508,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ((), ['hi'], ()))
 
     def test_scalar_set_commit_reset(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.someattr = 'hi'
@@ -1201,7 +1517,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), (['there'], (), ['hi']))
 
     def test_scalar_set_commit_reset_commit(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.someattr = 'hi'
@@ -1211,7 +1527,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ((), ['there'], ()))
 
     def test_scalar_set_commit_reset_commit_del(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.someattr = 'there'
@@ -1220,14 +1536,14 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ((), (), ['there']))
 
     def test_scalar_set_dict(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.__dict__['someattr'] = 'new'
         eq_(self._someattr_history(f), ((), ['new'], ()))
 
     def test_scalar_set_dict_set(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.__dict__['someattr'] = 'new'
@@ -1236,7 +1552,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), (['old'], (), ['new']))
 
     def test_scalar_set_dict_set_commit(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.__dict__['someattr'] = 'new'
@@ -1246,14 +1562,14 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ((), ['old'], ()))
 
     def test_scalar_set_None(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.someattr = None
         eq_(self._someattr_history(f), ([None], (), ()))
 
     def test_scalar_set_None_from_dict_set(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.__dict__['someattr'] = 'new'
@@ -1261,7 +1577,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ([None], (), ['new']))
 
     def test_scalar_set_twice_no_commit(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.someattr = 'one'
@@ -1270,13 +1586,13 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), (['two'], (), ()))
 
     def test_scalar_active_init(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=True)
         f = Foo()
         eq_(self._someattr_history(f), ((), (), ()))
 
     def test_scalar_active_no_init_side_effect(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=True)
         f = Foo()
         self._someattr_history(f)
@@ -1284,15 +1600,21 @@ class HistoryTest(fixtures.TestBase):
         assert 'someattr' not in f.__dict__
         assert 'someattr' not in attributes.instance_state(f).committed_state
 
+    def test_collection_never_set(self):
+        Foo = self._fixture(uselist=True, useobject=True,
+                                active_history=True)
+        f = Foo()
+        eq_(self._someattr_history(f, passive=True), ((), [], ()))
+
     def test_scalar_active_set(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=True)
         f = Foo()
         f.someattr = 'hi'
         eq_(self._someattr_history(f), (['hi'], (), ()))
 
     def test_scalar_active_set_commit(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=True)
         f = Foo()
         f.someattr = 'hi'
@@ -1300,7 +1622,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ((), ['hi'], ()))
 
     def test_scalar_active_set_commit_reset(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=True)
         f = Foo()
         f.someattr = 'hi'
@@ -1309,7 +1631,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), (['there'], (), ['hi']))
 
     def test_scalar_active_set_commit_reset_commit(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=True)
         f = Foo()
         f.someattr = 'hi'
@@ -1319,7 +1641,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ((), ['there'], ()))
 
     def test_scalar_active_set_commit_reset_commit_del(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=True)
         f = Foo()
         f.someattr = 'there'
@@ -1328,14 +1650,14 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ((), (), ['there']))
 
     def test_scalar_active_set_dict(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=True)
         f = Foo()
         f.__dict__['someattr'] = 'new'
         eq_(self._someattr_history(f), ((), ['new'], ()))
 
     def test_scalar_active_set_dict_set(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=True)
         f = Foo()
         f.__dict__['someattr'] = 'new'
@@ -1344,7 +1666,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), (['old'], (), ['new']))
 
     def test_scalar_active_set_dict_set_commit(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=True)
         f = Foo()
         f.__dict__['someattr'] = 'new'
@@ -1354,14 +1676,14 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ((), ['old'], ()))
 
     def test_scalar_active_set_None(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=True)
         f = Foo()
         f.someattr = None
         eq_(self._someattr_history(f), ([None], (), ()))
 
     def test_scalar_active_set_None_from_dict_set(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=True)
         f = Foo()
         f.__dict__['someattr'] = 'new'
@@ -1369,7 +1691,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ([None], (), ['new']))
 
     def test_scalar_active_set_twice_no_commit(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=True)
         f = Foo()
         f.someattr = 'one'
@@ -1379,14 +1701,14 @@ class HistoryTest(fixtures.TestBase):
 
 
     def test_mutable_scalar_init(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False,
                                 mutable_scalars=True,copy_function=dict)
         f = Foo()
         eq_(self._someattr_history(f), ((), (), ()))
 
     def test_mutable_scalar_no_init_side_effect(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False,
                                 mutable_scalars=True,copy_function=dict)
         f = Foo()
@@ -1395,7 +1717,7 @@ class HistoryTest(fixtures.TestBase):
         assert 'someattr' not in attributes.instance_state(f).committed_state
 
     def test_mutable_scalar_set(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False,
                                 mutable_scalars=True,copy_function=dict)
         f = Foo()
@@ -1403,7 +1725,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ([{'foo': 'hi'}], (), ()))
 
     def test_mutable_scalar_set_commit(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False,
                                 mutable_scalars=True,copy_function=dict)
         f = Foo()
@@ -1414,7 +1736,7 @@ class HistoryTest(fixtures.TestBase):
             {'foo': 'hi'})
 
     def test_mutable_scalar_set_commit_reset(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False,
                                 mutable_scalars=True,copy_function=dict)
         f = Foo()
@@ -1426,7 +1748,7 @@ class HistoryTest(fixtures.TestBase):
             'someattr'), ([{'foo': 'there'}], (), [{'foo': 'hi'}]))
 
     def test_mutable_scalar_set_commit_reset_commit(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False,
                                 mutable_scalars=True,copy_function=dict)
         f = Foo()
@@ -1437,7 +1759,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ((), [{'foo': 'there'}], ()))
 
     def test_mutable_scalar_set_dict(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False,
                                 mutable_scalars=True,copy_function=dict)
         f = Foo()
@@ -1445,7 +1767,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ((), [{'foo': 'new'}], ()))
 
     def test_mutable_scalar_set_dict_set(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False,
                                 mutable_scalars=True,copy_function=dict)
         f = Foo()
@@ -1455,7 +1777,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ([{'foo': 'old'}], (), [{'foo': 'new'}]))
 
     def test_mutable_scalar_set_dict_set_commit(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False,
                                 mutable_scalars=True,copy_function=dict)
         f = Foo()
@@ -1465,14 +1787,14 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ((), [{'foo': 'old'}], ()))
 
     def test_scalar_inplace_mutation_set(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.someattr = {'a': 'b'}
         eq_(self._someattr_history(f), ([{'a': 'b'}], (), ()))
 
     def test_scalar_inplace_mutation_set_commit(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.someattr = {'a': 'b'}
@@ -1480,7 +1802,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ((), [{'a': 'b'}], ()))
 
     def test_scalar_inplace_mutation_set_commit_set(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.someattr = {'a': 'b'}
@@ -1489,7 +1811,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ((), [{'a': 'c'}], ()))
 
     def test_scalar_inplace_mutation_set_commit_flag_modified(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.someattr = {'a': 'b'}
@@ -1498,7 +1820,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ([{'a': 'b'}], (), ()))
 
     def test_scalar_inplace_mutation_set_commit_set_flag_modified(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.someattr = {'a': 'b'}
@@ -1508,7 +1830,7 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), ([{'a': 'c'}], (), ()))
 
     def test_scalar_inplace_mutation_set_commit_flag_modified_set(self):
-        Foo = self._fixture(uselist=False, useobject=False, 
+        Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
         f = Foo()
         f.someattr = {'a': 'b'}
@@ -2189,8 +2511,8 @@ class ListenerTest(fixtures.ORMTest):
             (make_a, make_b),
             (make_b, make_c)
         ]
-        elements = [make_a, make_b, make_c, 
-                    instrument_a, instrument_b, instrument_c, 
+        elements = [make_a, make_b, make_c,
+                    instrument_a, instrument_b, instrument_c,
                     attr_a, attr_b, attr_c, events_a]
 
         for i, series in enumerate(all_partial_orderings(ordering, elements)):
