@@ -12,14 +12,13 @@ from sqlalchemy import schema, exc
 import sqlalchemy as tsa
 from test.lib import fixtures
 from test.lib import testing
-from test.lib import engines
 from test.lib.testing import ComparesTables, AssertsCompiledSQL
-from test.lib.testing import eq_
+from test.lib.testing import eq_, is_
 
 class MetaDataTest(fixtures.TestBase, ComparesTables):
     def test_metadata_connect(self):
         metadata = MetaData()
-        t1 = Table('table1', metadata, 
+        t1 = Table('table1', metadata,
             Column('col1', Integer, primary_key=True),
             Column('col2', String(20)))
         metadata.bind = testing.db
@@ -58,7 +57,7 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
             Column('bar', Integer(), info={'foo':'bar'}),
         ]:
             c2 = col.copy()
-            for attr in ('name', 'type', 'nullable', 
+            for attr in ('name', 'type', 'nullable',
                         'primary_key', 'key', 'unique', 'info',
                         'doc'):
                 eq_(getattr(col, attr), getattr(c2, attr))
@@ -96,15 +95,15 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
             # that listeners will be re-established from the
             # natural construction of things.
             cx._on_table_attach(write)
-            t = Table('foo%d' % i, m, cx)
+            Table('foo%d' % i, m, cx)
         eq_(msgs, ['attach foo0.foo', 'attach foo1.foo', 'attach foo2.foo'])
 
     def test_schema_collection_add(self):
         metadata = MetaData()
 
-        t1 = Table('t1', metadata, Column('x', Integer), schema='foo')
-        t2 = Table('t2', metadata, Column('x', Integer), schema='bar')
-        t3 = Table('t3', metadata, Column('x', Integer))
+        Table('t1', metadata, Column('x', Integer), schema='foo')
+        Table('t2', metadata, Column('x', Integer), schema='bar')
+        Table('t3', metadata, Column('x', Integer))
 
         eq_(metadata._schemas, set(['foo', 'bar']))
         eq_(len(metadata.tables), 3)
@@ -113,7 +112,7 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
         metadata = MetaData()
 
         t1 = Table('t1', metadata, Column('x', Integer), schema='foo')
-        t2 = Table('t2', metadata, Column('x', Integer), schema='bar')
+        Table('t2', metadata, Column('x', Integer), schema='bar')
         t3 = Table('t3', metadata, Column('x', Integer), schema='bar')
 
         metadata.remove(t3)
@@ -127,8 +126,8 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
     def test_schema_collection_remove_all(self):
         metadata = MetaData()
 
-        t1 = Table('t1', metadata, Column('x', Integer), schema='foo')
-        t2 = Table('t2', metadata, Column('x', Integer), schema='bar')
+        Table('t1', metadata, Column('x', Integer), schema='foo')
+        Table('t2', metadata, Column('x', Integer), schema='bar')
 
         metadata.clear()
         eq_(metadata._schemas, set())
@@ -137,7 +136,7 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
     def test_metadata_tables_immutable(self):
         metadata = MetaData()
 
-        t1 = Table('t1', metadata, Column('x', Integer))
+        Table('t1', metadata, Column('x', Integer))
         assert 't1' in metadata.tables
 
         assert_raises(
@@ -148,14 +147,14 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
     @testing.provide_metadata
     def test_dupe_tables(self):
         metadata = self.metadata
-        t1 = Table('table1', metadata, 
+        Table('table1', metadata,
             Column('col1', Integer, primary_key=True),
             Column('col2', String(20)))
 
         metadata.create_all()
-        t1 = Table('table1', metadata, autoload=True)
+        Table('table1', metadata, autoload=True)
         def go():
-            t2 = Table('table1', metadata, 
+            Table('table1', metadata,
                 Column('col1', Integer, primary_key=True),
                 Column('col2', String(20)))
         assert_raises_message(
@@ -173,11 +172,11 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
         m = MetaData()
         t1 = Table('t', m, c1, c2)
 
-        kw = dict(onupdate="X", 
+        kw = dict(onupdate="X",
                         ondelete="Y", use_alter=True, name='f1',
                         deferrable="Z", initially="Q", link_to_name=True)
 
-        fk1 = ForeignKey(c1, **kw) 
+        fk1 = ForeignKey(c1, **kw)
         fk2 = ForeignKeyConstraint((c1,), (c2,), **kw)
 
         t1.append_constraint(fk2)
@@ -190,11 +189,11 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
 
     def test_check_constraint_copy(self):
         r = lambda x: x
-        c = CheckConstraint("foo bar", 
-                            name='name', 
-                            initially=True, 
-                            deferrable=True, 
-                            _create_rule = r)
+        c = CheckConstraint("foo bar",
+                            name='name',
+                            initially=True,
+                            deferrable=True,
+                            _create_rule=r)
         c2 = c.copy()
         eq_(c2.name, 'name')
         eq_(str(c2.sqltext), "foo bar")
@@ -224,7 +223,7 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
     def test_fk_no_such_parent_col_error(self):
         meta = MetaData()
         a = Table('a', meta, Column('a', Integer))
-        b = Table('b', meta, Column('b', Integer))
+        Table('b', meta, Column('b', Integer))
 
         def go():
             a.append_constraint(
@@ -240,7 +239,7 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
     def test_fk_no_such_target_col_error(self):
         meta = MetaData()
         a = Table('a', meta, Column('a', Integer))
-        b = Table('b', meta, Column('b', Integer))
+        Table('b', meta, Column('b', Integer))
         a.append_constraint(
             ForeignKeyConstraint(['a'], ['b.x'])
         )
@@ -273,7 +272,7 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
 
         table2 = Table('othertable', meta,
             Column('id', Integer, Sequence('foo_seq'), primary_key=True),
-            Column('myid', Integer, 
+            Column('myid', Integer,
                         ForeignKey('mytable.myid'),
                     ),
             test_needs_fk=True,
@@ -289,7 +288,7 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
             meta.bind = testing.db
             meta2 = pickle.loads(pickle.dumps(meta))
             assert meta2.bind is None
-            meta3 = pickle.loads(pickle.dumps(meta2))
+            pickle.loads(pickle.dumps(meta2))
             return (meta2.tables['mytable'], meta2.tables['othertable'])
 
         def test_pickle_via_reflect():
@@ -297,7 +296,7 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
             # database reflection
             meta2 = MetaData(bind=testing.db)
             t1 = Table('mytable', meta2, autoload=True)
-            t2 = Table('othertable', meta2, autoload=True)
+            Table('othertable', meta2, autoload=True)
             meta3 = pickle.loads(pickle.dumps(meta2))
             assert meta3.bind is None
             assert meta3.tables['mytable'] is not t1
@@ -350,17 +349,29 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
         finally:
             meta.drop_all(testing.db)
 
+    def test_col_key_fk_parent_tometadata(self):
+        # test #2643
+        m1 = MetaData()
+        a = Table('a', m1, Column('x', Integer))
+        b = Table('b', m1, Column('x', Integer, ForeignKey('a.x'), key='y'))
+        assert b.c.y.references(a.c.x)
+
+        m2 = MetaData()
+        b2 = b.tometadata(m2)
+        a2 = a.tometadata(m2)
+        assert b2.c.y.references(a2.c.x)
+
     def test_pickle_metadata_sequence_restated(self):
         m1 = MetaData()
-        Table('a',m1,
-             Column('id',Integer,primary_key=True),
+        Table('a', m1,
+             Column('id', Integer, primary_key=True),
              Column('x', Integer, Sequence("x_seq")))
 
         m2 = pickle.loads(pickle.dumps(m1))
 
         s2 = Sequence("x_seq")
-        t2 = Table('a', m2, 
-             Column('id',Integer,primary_key=True),
+        t2 = Table('a', m2,
+             Column('id', Integer, primary_key=True),
              Column('x', Integer, s2),
              extend_existing=True)
 
@@ -373,13 +384,13 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
 
         m1 = MetaData()
         s1 = Sequence("x_seq")
-        t = Table('a', m1, 
+        t = Table('a', m1,
              Column('x', Integer, s1)
         )
         assert m1._sequences['x_seq'] is s1
 
         s2 = Sequence('x_seq')
-        t2 = Table('a', m1,
+        Table('a', m1,
              Column('x', Integer, s2),
              extend_existing=True
         )
@@ -389,26 +400,26 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
 
     def test_pickle_metadata_sequence_implicit(self):
         m1 = MetaData()
-        Table('a',m1,
-             Column('id',Integer,primary_key=True),
+        Table('a', m1,
+             Column('id', Integer, primary_key=True),
              Column('x', Integer, Sequence("x_seq")))
 
         m2 = pickle.loads(pickle.dumps(m1))
 
         t2 = Table('a', m2, extend_existing=True)
 
-        eq_(m2._sequences, {'x_seq':t2.c.x.default})
+        eq_(m2._sequences, {'x_seq': t2.c.x.default})
 
     def test_pickle_metadata_schema(self):
         m1 = MetaData()
-        Table('a',m1,
-             Column('id',Integer,primary_key=True),
+        Table('a', m1,
+             Column('id', Integer, primary_key=True),
              Column('x', Integer, Sequence("x_seq")),
              schema='y')
 
         m2 = pickle.loads(pickle.dumps(m1))
 
-        t2 = Table('a', m2, schema='y',
+        Table('a', m2, schema='y',
              extend_existing=True)
 
         eq_(m2._schemas, m1._schemas)
@@ -481,7 +492,7 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
         meta2 = MetaData()
         table_c = table.tometadata(meta2)
 
-        eq_(table.kwargs,table_c.kwargs)
+        eq_(table.kwargs, table_c.kwargs)
 
     def test_tometadata_indexes(self):
         meta = MetaData()
@@ -491,13 +502,13 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
             Column('data1', Integer, index=True),
             Column('data2', Integer),
         )
-        Index('multi',table.c.data1,table.c.data2),
+        Index('multi', table.c.data1, table.c.data2),
 
         meta2 = MetaData()
         table_c = table.tometadata(meta2)
 
         def _get_key(i):
-            return [i.name,i.unique] + \
+            return [i.name, i.unique] + \
                     sorted(i.kwargs.items()) + \
                     i.columns.keys()
 
@@ -518,8 +529,6 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
             Column('yourid', Integer, primary_key=True),
         )
 
-        meta3 = MetaData()
-
         table_c = table1.tometadata(meta2)
         table_d = table2.tometadata(meta2)
 
@@ -532,7 +541,8 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
         m3 = MetaData(schema='sch1', quote_schema=False)
         m4 = MetaData()
 
-        for i, (name, metadata, schema, quote_schema, exp_schema, exp_quote_schema) in enumerate([
+        for i, (name, metadata, schema, quote_schema,
+                        exp_schema, exp_quote_schema) in enumerate([
             ('t1', m1, None, None, 'sch1', None),
             ('t2', m1, 'sch2', None, 'sch2', None),
             ('t3', m1, 'sch2', True, 'sch2', True),
@@ -601,9 +611,9 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
         table_c = table.tometadata(meta2, schema=None)
         table2_c = table2.tometadata(meta2, schema=None)
 
-        eq_(str(table_c.join(table2_c).onclause), 
+        eq_(str(table_c.join(table2_c).onclause),
                 str(table_c.c.myid == table2_c.c.myid))
-        eq_(str(table_c.join(table2_c).onclause), 
+        eq_(str(table_c.join(table2_c).onclause),
                 "someschema.mytable.myid = someschema.othertable.myid")
 
     def test_tometadata_strip_schema(self):
@@ -644,21 +654,21 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
         ck = schema.CheckConstraint("x > y", name="someconstraint")
 
         for const, exp in (
-            (Sequence("my_seq"), 
+            (Sequence("my_seq"),
                 "Sequence('my_seq')"),
-            (Sequence("my_seq", start=5), 
+            (Sequence("my_seq", start=5),
                 "Sequence('my_seq', start=5)"),
-            (Column("foo", Integer), 
+            (Column("foo", Integer),
                 "Column('foo', Integer(), table=None)"),
-            (Table("bar", MetaData(), Column("x", String)), 
+            (Table("bar", MetaData(), Column("x", String)),
                 "Table('bar', MetaData(bind=None), "
                 "Column('x', String(), table=<bar>), schema=None)"),
-            (schema.DefaultGenerator(for_update=True), 
+            (schema.DefaultGenerator(for_update=True),
                 "DefaultGenerator(for_update=True)"),
-            (schema.Index("bar"), "Index('bar')"),
+            (schema.Index("bar", "c"), "Index('bar')"),
             (i1, "Index('bar', Column('x', Integer(), table=<foo>))"),
             (schema.FetchedValue(), "FetchedValue()"),
-            (ck, 
+            (ck,
                     "CheckConstraint("
                     "%s"
                     ", name='someconstraint')" % repr(ck.sqltext)),
@@ -670,29 +680,30 @@ class MetaDataTest(fixtures.TestBase, ComparesTables):
 
 class TableTest(fixtures.TestBase, AssertsCompiledSQL):
     def test_prefixes(self):
+        from sqlalchemy import Table
         table1 = Table("temporary_table_1", MetaData(),
                       Column("col1", Integer),
-                      prefixes = ["TEMPORARY"])
+                      prefixes=["TEMPORARY"])
 
         self.assert_compile(
-            schema.CreateTable(table1), 
+            schema.CreateTable(table1),
             "CREATE TEMPORARY TABLE temporary_table_1 (col1 INTEGER)"
         )
 
         table2 = Table("temporary_table_2", MetaData(),
                       Column("col1", Integer),
-                      prefixes = ["VIRTUAL"])
+                      prefixes=["VIRTUAL"])
         self.assert_compile(
-          schema.CreateTable(table2), 
+          schema.CreateTable(table2),
           "CREATE VIRTUAL TABLE temporary_table_2 (col1 INTEGER)"
         )
 
     def test_table_info(self):
         metadata = MetaData()
-        t1 = Table('foo', metadata, info={'x':'y'})
+        t1 = Table('foo', metadata, info={'x': 'y'})
         t2 = Table('bar', metadata, info={})
         t3 = Table('bat', metadata)
-        assert t1.info == {'x':'y'}
+        assert t1.info == {'x': 'y'}
         assert t2.info == {}
         assert t3.info == {}
         for t in (t1, t2, t3):
@@ -714,13 +725,28 @@ class TableTest(fixtures.TestBase, AssertsCompiledSQL):
             assign
         )
 
-        def assign():
+        def assign2():
             t1.c.z = Column('z', Integer)
         assert_raises(
             TypeError,
-            assign
+            assign2
         )
 
+    def test_autoincrement_replace(self):
+        m = MetaData()
+
+        t = Table('t', m,
+            Column('id', Integer, primary_key=True)
+        )
+
+        is_(t._autoincrement_column, t.c.id)
+
+        t = Table('t', m,
+            Column('id', Integer, primary_key=True),
+            extend_existing=True
+        )
+        is_(t._autoincrement_column, t.c.id)
+
 class SchemaTest(fixtures.TestBase, AssertsCompiledSQL):
 
     def test_default_schema_metadata_fk(self):
@@ -741,7 +767,7 @@ class SchemaTest(fixtures.TestBase, AssertsCompiledSQL):
     def test_default_schema_metadata_fk_alt_remote(self):
         m = MetaData(schema="foo")
         t1 = Table('t1', m, Column('x', Integer))
-        t2 = Table('t2', m, Column('x', Integer, ForeignKey('t1.x')), 
+        t2 = Table('t2', m, Column('x', Integer, ForeignKey('t1.x')),
                                 schema="bar")
         assert t2.c.x.references(t1.c.x)
 
@@ -797,8 +823,8 @@ class SchemaTest(fixtures.TestBase, AssertsCompiledSQL):
 class UseExistingTest(fixtures.TablesTest):
     @classmethod
     def define_tables(cls, metadata):
-        Table('users', metadata, 
-                    Column('id', Integer, primary_key=True), 
+        Table('users', metadata,
+                    Column('id', Integer, primary_key=True),
                     Column('name', String(30)))
 
     def _useexisting_fixture(self):
@@ -812,7 +838,7 @@ class UseExistingTest(fixtures.TablesTest):
     def test_exception_no_flags(self):
         meta2 = self._useexisting_fixture()
         def go():
-            users = Table('users', meta2, Column('name',
+            Table('users', meta2, Column('name',
                           Unicode), autoload=True)
         assert_raises_message(
             exc.InvalidRequestError,
@@ -836,7 +862,7 @@ class UseExistingTest(fixtures.TablesTest):
         meta2 = self._useexisting_fixture()
         assert_raises(
             exc.ArgumentError,
-            Table, 'users', meta2, keep_existing=True, 
+            Table, 'users', meta2, keep_existing=True,
                 extend_existing=True
         )
 
@@ -845,13 +871,13 @@ class UseExistingTest(fixtures.TablesTest):
         meta2 = self._useexisting_fixture()
         assert_raises(
             exc.ArgumentError,
-            Table, 'users', meta2, useexisting=True, 
+            Table, 'users', meta2, useexisting=True,
                 extend_existing=True
         )
 
     def test_keep_existing_no_dupe_constraints(self):
         meta2 = self._notexisting_fixture()
-        users = Table('users', meta2, 
+        users = Table('users', meta2,
             Column('id', Integer),
             Column('name', Unicode),
             UniqueConstraint('name'),
@@ -861,7 +887,7 @@ class UseExistingTest(fixtures.TablesTest):
         assert 'id' in users.c
         eq_(len(users.constraints), 2)
 
-        u2 = Table('users', meta2, 
+        u2 = Table('users', meta2,
             Column('id', Integer),
             Column('name', Unicode),
             UniqueConstraint('name'),
@@ -871,7 +897,7 @@ class UseExistingTest(fixtures.TablesTest):
 
     def test_extend_existing_dupes_constraints(self):
         meta2 = self._notexisting_fixture()
-        users = Table('users', meta2, 
+        users = Table('users', meta2,
             Column('id', Integer),
             Column('name', Unicode),
             UniqueConstraint('name'),
@@ -881,7 +907,7 @@ class UseExistingTest(fixtures.TablesTest):
         assert 'id' in users.c
         eq_(len(users.constraints), 2)
 
-        u2 = Table('users', meta2, 
+        u2 = Table('users', meta2,
             Column('id', Integer),
             Column('name', Unicode),
             UniqueConstraint('name'),
@@ -904,7 +930,7 @@ class UseExistingTest(fixtures.TablesTest):
 
     def test_keep_existing_add_column(self):
         meta2 = self._useexisting_fixture()
-        users = Table('users', meta2, 
+        users = Table('users', meta2,
                         Column('foo', Integer),
                         autoload=True,
                       keep_existing=True)
@@ -918,14 +944,14 @@ class UseExistingTest(fixtures.TablesTest):
 
     def test_keep_existing_quote_no_orig(self):
         meta2 = self._notexisting_fixture()
-        users = Table('users', meta2, quote=True, 
+        users = Table('users', meta2, quote=True,
                         autoload=True,
                       keep_existing=True)
         assert users.quote
 
     def test_keep_existing_add_column_no_orig(self):
         meta2 = self._notexisting_fixture()
-        users = Table('users', meta2, 
+        users = Table('users', meta2,
                         Column('foo', Integer),
                         autoload=True,
                       keep_existing=True)
@@ -939,13 +965,13 @@ class UseExistingTest(fixtures.TablesTest):
 
     def test_keep_existing_quote_no_reflection(self):
         meta2 = self._useexisting_fixture()
-        users = Table('users', meta2, quote=True, 
+        users = Table('users', meta2, quote=True,
                       keep_existing=True)
         assert not users.quote
 
     def test_keep_existing_add_column_no_reflection(self):
         meta2 = self._useexisting_fixture()
-        users = Table('users', meta2, 
+        users = Table('users', meta2,
                         Column('foo', Integer),
                       keep_existing=True)
         assert "foo" not in users.c
@@ -964,7 +990,7 @@ class UseExistingTest(fixtures.TablesTest):
 
     def test_extend_existing_add_column(self):
         meta2 = self._useexisting_fixture()
-        users = Table('users', meta2, 
+        users = Table('users', meta2,
                         Column('foo', Integer),
                         autoload=True,
                       extend_existing=True)
@@ -978,14 +1004,14 @@ class UseExistingTest(fixtures.TablesTest):
 
     def test_extend_existing_quote_no_orig(self):
         meta2 = self._notexisting_fixture()
-        users = Table('users', meta2, quote=True, 
+        users = Table('users', meta2, quote=True,
                         autoload=True,
                       extend_existing=True)
         assert users.quote
 
     def test_extend_existing_add_column_no_orig(self):
         meta2 = self._notexisting_fixture()
-        users = Table('users', meta2, 
+        users = Table('users', meta2,
                         Column('foo', Integer),
                         autoload=True,
                       extend_existing=True)
@@ -999,13 +1025,13 @@ class UseExistingTest(fixtures.TablesTest):
 
     def test_extend_existing_quote_no_reflection(self):
         meta2 = self._useexisting_fixture()
-        users = Table('users', meta2, quote=True, 
+        users = Table('users', meta2, quote=True,
                       extend_existing=True)
         assert users.quote
 
     def test_extend_existing_add_column_no_reflection(self):
         meta2 = self._useexisting_fixture()
-        users = Table('users', meta2, 
+        users = Table('users', meta2,
                         Column('foo', Integer),
                       extend_existing=True)
         assert "foo" in users.c
@@ -1014,16 +1040,16 @@ class ConstraintTest(fixtures.TestBase):
     def _single_fixture(self):
         m = MetaData()
 
-        t1 = Table('t1', m, 
+        t1 = Table('t1', m,
             Column('a', Integer),
             Column('b', Integer)
         )
 
-        t2 = Table('t2', m, 
+        t2 = Table('t2', m,
             Column('a', Integer, ForeignKey('t1.a'))
         )
 
-        t3 = Table('t3', m, 
+        t3 = Table('t3', m,
             Column('a', Integer)
         )
         return t1, t2, t3
@@ -1063,7 +1089,7 @@ class ColumnDefinitionTest(AssertsCompiledSQL, fixtures.TestBase):
     __dialect__ = 'default'
 
     def columns(self):
-        return [ Column(Integer),
+        return [Column(Integer),
                  Column('b', Integer),
                  Column(Integer),
                  Column('d', Integer),
@@ -1071,7 +1097,7 @@ class ColumnDefinitionTest(AssertsCompiledSQL, fixtures.TestBase):
                  Column(type_=Integer),
                  Column(Integer()),
                  Column('h', Integer()),
-                 Column(type_=Integer()) ]
+                 Column(type_=Integer())]
 
     def test_basic(self):
         c = self.columns()
@@ -1090,7 +1116,7 @@ class ColumnDefinitionTest(AssertsCompiledSQL, fixtures.TestBase):
 
         c = Column(Integer)
         assert_raises_message(
-            exc.ArgumentError, 
+            exc.ArgumentError,
             "Column must be constructed with a non-blank name or assign a "
             "non-blank .name ",
             Table, 't', MetaData(), c)
@@ -1099,17 +1125,17 @@ class ColumnDefinitionTest(AssertsCompiledSQL, fixtures.TestBase):
 
         c = Column('', Integer)
         assert_raises_message(
-            exc.ArgumentError, 
+            exc.ArgumentError,
             "Column must be constructed with a non-blank name or assign a "
             "non-blank .name ",
             Table, 't', MetaData(), c)
 
     def test_dupe_column(self):
         c = Column('x', Integer)
-        t = Table('t', MetaData(), c)
+        Table('t', MetaData(), c)
 
         assert_raises_message(
-            exc.ArgumentError, 
+            exc.ArgumentError,
             "Column object already assigned to Table 't'",
             Table, 'q', MetaData(), c)
 
@@ -1119,7 +1145,7 @@ class ColumnDefinitionTest(AssertsCompiledSQL, fixtures.TestBase):
         assert c.key is None
 
         c.name = 'named'
-        t = Table('t', MetaData(), c)
+        Table('t', MetaData(), c)
 
         assert c.name == 'named'
         assert c.name == c.key
@@ -1198,7 +1224,7 @@ class ColumnDefinitionTest(AssertsCompiledSQL, fixtures.TestBase):
         assert_raises_message(
             TypeError,
             "Could not create a copy of this <class "
-            "'test.sql.test_metadata.MyColumn'> "
+            "'test.sql.test_metadata..*MyColumn'> "
             "object.  Ensure the class includes a _constructor()",
             getattr, select([t1.select().alias()]), 'c'
         )
@@ -1215,6 +1241,37 @@ class ColumnDefaultsTest(fixtures.TestBase):
         assert c.server_default is target
         assert target.column is c
 
+    def test_onupdate_default_not_server_default_one(self):
+        target1 = schema.DefaultClause('y')
+        target2 = schema.DefaultClause('z')
+
+        c = self._fixture(server_default=target1, server_onupdate=target2)
+        eq_(c.server_default.arg, 'y')
+        eq_(c.server_onupdate.arg, 'z')
+
+    def test_onupdate_default_not_server_default_two(self):
+        target1 = schema.DefaultClause('y', for_update=True)
+        target2 = schema.DefaultClause('z', for_update=True)
+
+        c = self._fixture(server_default=target1, server_onupdate=target2)
+        eq_(c.server_default.arg, 'y')
+        eq_(c.server_onupdate.arg, 'z')
+
+    def test_onupdate_default_not_server_default_three(self):
+        target1 = schema.DefaultClause('y', for_update=False)
+        target2 = schema.DefaultClause('z', for_update=True)
+
+        c = self._fixture(target1, target2)
+        eq_(c.server_default.arg, 'y')
+        eq_(c.server_onupdate.arg, 'z')
+
+    def test_onupdate_default_not_server_default_four(self):
+        target1 = schema.DefaultClause('y', for_update=False)
+
+        c = self._fixture(server_onupdate=target1)
+        is_(c.server_default, None)
+        eq_(c.server_onupdate.arg, 'y')
+
     def test_server_default_keyword_as_schemaitem(self):
         target = schema.DefaultClause('y')
         c = self._fixture(server_default=target)
@@ -1306,10 +1363,10 @@ class ColumnOptionsTest(fixtures.TestBase):
 
     def test_column_info(self):
 
-        c1 = Column('foo', String, info={'x':'y'})
+        c1 = Column('foo', String, info={'x': 'y'})
         c2 = Column('bar', String, info={})
         c3 = Column('bat', String)
-        assert c1.info == {'x':'y'}
+        assert c1.info == {'x': 'y'}
         assert c2.info == {}
         assert c3.info == {}
 
@@ -1335,11 +1392,11 @@ class CatchAllEventsTest(fixtures.TestBase):
         event.listen(schema.SchemaItem, "after_parent_attach", after_attach)
 
         m = MetaData()
-        t1 = Table('t1', m, 
+        Table('t1', m,
             Column('id', Integer, Sequence('foo_id'), primary_key=True),
             Column('bar', String, ForeignKey('t2.id'))
         )
-        t2 = Table('t2', m,
+        Table('t2', m,
             Column('id', Integer, primary_key=True),
         )
 
@@ -1375,12 +1432,12 @@ class CatchAllEventsTest(fixtures.TestBase):
             evt(target)
 
         m = MetaData()
-        t1 = Table('t1', m, 
+        Table('t1', m,
             Column('id', Integer, Sequence('foo_id'), primary_key=True),
             Column('bar', String, ForeignKey('t2.id')),
             Column('bat', Integer, unique=True),
         )
-        t2 = Table('t2', m,
+        Table('t2', m,
             Column('id', Integer, primary_key=True),
             Column('bar', Integer),
             Column('bat', Integer),
@@ -1390,10 +1447,10 @@ class CatchAllEventsTest(fixtures.TestBase):
         eq_(
             canary,
             [
-            'PrimaryKeyConstraint->Table', 'PrimaryKeyConstraint->t1', 
+            'PrimaryKeyConstraint->Table', 'PrimaryKeyConstraint->t1',
             'ForeignKeyConstraint->Table', 'ForeignKeyConstraint->t1',
             'UniqueConstraint->Table', 'UniqueConstraint->t1',
-            'PrimaryKeyConstraint->Table', 'PrimaryKeyConstraint->t2', 
+            'PrimaryKeyConstraint->Table', 'PrimaryKeyConstraint->t2',
             'CheckConstraint->Table', 'CheckConstraint->t2',
             'UniqueConstraint->Table', 'UniqueConstraint->t2'
             ]
